/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor2;

import java.awt.Dimension;

import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

/**
 * Add a rows to a table.
 * 
 * @author jkaisinger
 *
 */
public class RowHeaderTable extends JTable {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Local reference of the JTable. It allows to manipulate the JTable.
	 */
	private JTable table;

	/**
	 * Creates an object.
	 * 
	 * @param mainTable
	 *            JTable
	 */
	public RowHeaderTable(JTable mainTable) {
		super(new DefaultTableModel());
		this.createRowTable(mainTable);
	}

	/**
	 * Creates the RowHeaderTable.
	 * 
	 * @param mainTable
	 */
	private void createRowTable(JTable mainTable) {
		this.table = mainTable;

		DefaultTableModel rowHeadersModel = (DefaultTableModel) this.getModel();
		rowHeadersModel.addColumn("");

		for (int r = 0; r < this.table.getRowCount(); r++) {
			rowHeadersModel.addRow(new Object[] { "" + r });
		}

		// center the row number
		DefaultTableCellRenderer rightRenderer = new DefaultTableCellRenderer();
		rightRenderer.setHorizontalAlignment(SwingConstants.CENTER);
		this.getColumnModel().getColumn(0).setCellRenderer(rightRenderer);

		// this set the width for the row number column
		this.getColumnModel().getColumn(0).setPreferredWidth(25);
		Dimension d = this.getPreferredScrollableViewportSize();
		d.width = this.getPreferredSize().width;
		this.setPreferredScrollableViewportSize(d);

		// make the rows look like headers
		this.getTableHeader().setBackground(this.table.getTableHeader().getBackground());
		this.getTableHeader().setForeground(this.table.getTableHeader().getForeground());

		this.setBackground(this.getTableHeader().getBackground());
		this.setForeground(this.getTableHeader().getForeground());
		this.setFont(this.getTableHeader().getFont());

		this.setEnabled(false);
		this.getTableHeader().setReorderingAllowed(false);

		this.table.getModel().addTableModelListener(new TableModelListener() {
			@Override
			public void tableChanged(TableModelEvent e) {
				RowHeaderTable.this.updateRows();
			}
		});
	}

	/**
	 * Update the rows.
	 */
	public void updateRows() {
		DefaultTableModel rowHeadersModel = (DefaultTableModel) this.getModel();

		if (this.getRowCount() != this.table.getRowCount()) {

			if (this.getRowCount() > this.table.getRowCount()) {
				rowHeadersModel.removeRow(0);
			} else {
				rowHeadersModel.insertRow(0, new Object[] { "" + 0 });
			}

			for (int r = 0; r < rowHeadersModel.getRowCount(); r++) {
				rowHeadersModel.setValueAt(r, r, 0);
			}
		}
	}

}
