#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 957 $
$LastChangedDate:: 2019-06-13 13:25:47 +0200#$

This file is part of SyMSpace.

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import numpy as np
from lcm.tdata_interface.pyjavadata_interface import PyJavaDataInterface


class PyJListInterface(PyJavaDataInterface):
    """Java ArrayList conversion

    Java                    | Python
    --------------------------------
    ArrayList/jep.PyJList   | numpy.ndarray with numpy.dtype(object)

    """

    @staticmethod
    def convert_J2Py(jobj):
        """
        Return a Python object of type "np.array" with "dtype=np.dtype(object)"
        equivalent to given Java object of type "PyJList"
        """
        from lcm.tdata_interface.pyjdata_interface import PyJDataInterface

        '''initialize pyobj if jobj.size() == 0'''
        pyobj = np.array([], dtype=np.dtype(object))
        for index in range(0, jobj.size()):
            temp_pyobj = PyJDataInterface.convert_J2Py(jobj[index])
            if index == 0:
                pyobj = np.array([temp_pyobj], dtype=np.dtype(object))
            else:
                pyobj = np.append(pyobj, [temp_pyobj], axis=0)
        return pyobj

    @staticmethod
    def convert_Py2J(pyobj):
        """
        Return a Java object of type "jep.PyJList"
        equivalent to given python object of type "list"
        """

        raise NotImplementedError("Conversion of a python object", type(pyobj),
                                  "to a  Java object of type 'jep.PyJArray' has not been implemented.")
        # return jobj


if __name__ == "__main__":
    # noinspection PyUnresolvedReferences
    from java.util import ArrayList
    from lcm.tdata_interface.pyjdata_interface import PyJDataInterface

    COUNT = 17

    def makeJavaList():
        jlist = ArrayList()
        for i in range(COUNT):
            jlist.add(i)
            jlist.add('str')
        return jlist

    tmp_jobj = makeJavaList()
    tmp_pyobj = PyJDataInterface.convert_J2Py(tmp_jobj)
    print('java:' + str(tmp_jobj))
    print('python:' + str(tmp_pyobj))
    print('python type:' + str(type(tmp_pyobj)))
