"""
Created on 10.08.2016 14:01

@author: kkaspar
"""

import numpy as np


def is_integer(var):
    """
    Check if the argument is an integer number

    Parameters
    ----------
    var : int, float

    Returns
    -------
    bool
        True if mathematical integer number else False
    """
    try:
        if var % 1 == 0:
            return True
        else:
            return False
    except:
        return False


def is_array_like(var):
    """
    Checks if the input is an array_like type e.g: list, tuple or numpy.ndarray

    Parameters
    ----------
    var : array_like

    Returns
    -------
    bool
        True if array_like object
    """
    if isinstance(var, list) or isinstance(var, tuple) or isinstance(var, np.ndarray):
        return True
    else:
        return False


def get_integer_input(var, var_name):
    """
    Checks if the input variable is of type integer
    If the input variable is of type e.g. float, but divisible without remainder it is typecast to an integer

    Parameters
    ----------
    var: int, float, array_like
    var_name: string
              variable name

    Returns
    -------
    int
        Integer variable
    """
    from lcm.utils.common_fnc.array_fnc import array_like_to_float

    var = array_like_to_float(var, var_name)
    if is_integer(var):
        return int(var)
    else:
        raise ValueError('Input parameter', var_name, 'needs to be an integer value')


def get_float_input(var, var_name):
    """
    Checks if the input variable is of type float

    Parameters
    ----------
    var: int, float, array_like
    var_name: string
              variable name

    Returns
    -------
    float
        float variable
    """
    from lcm.utils.common_fnc.array_fnc import array_like_to_float

    var = array_like_to_float(var, var_name)
    if isinstance(var, float):
        return float(var)
    else:
        raise ValueError('Input parameter', var_name, 'needs to be a float (or integer) value')


def get_bool_input(var, var_name):
    """
    Checks if the input variable is of type bool

    Parameters
    ----------
    var: bool
    var_name: string
              variable name

    Returns
    -------
    bool
        Boolean variable
    """
    if isinstance(var, bool):
        return bool(var)
    elif is_integer(var) or isinstance(var, int):
        return bool(var)
    else:
        raise ValueError('Input parameter', var_name, 'needs to be a boolean')


def get_string_input(var, var_name):
    """
    Checks if the input variable is of type string

    Parameters
    ----------
    var : string
    var_name : string
               variable name

    Returns
    -------
    string
          String variable
    """
    if isinstance(var, str):
        return str(var)
    else:
        raise ValueError('Input parameter', var_name, 'needs to be a string')


def get_ndarray_input(var, var_name):
    """
    Checks if the input variable is array_like and converts to a numpy.ndarray if needed

    Parameters
    ----------
    var : numpy.ndarray
    var_name : string
                variable name
    Returns
    -------
    numpy.ndarray
         NDArray variable
    """
    if isinstance(var, np.ndarray):
        return var
    elif isinstance(var, list) or isinstance(var, tuple):
        return np.array(var)
    else:
        raise ValueError('Input parameter', var_name, 'needs to be a numpy ndarray')
