#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 1015 $
$LastChangedDate:: 2020-04-17 11:42:43 +0200#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import Ellipse


def getDataSet2D(index):
    if index == 0:
        '''outside max_dist = 0.5
        equidistant grid in x and y direction
        '''
        xc = np.array([[0, 0], [0, 0.5], [0, 1],
                       [0.5, 0], [0.5, 0.5], [0.5, 1],
                       [1, 0], [1, 0.5], [1, 1]]).transpose() - 0.5
        x = np.array([[0., 1.15]]).transpose()
    elif index == 1:
        '''inside max_dist = 0.5
        equidistant grid in x-direction only
        '''
        xc = np.array([[0, 0], [0, 0.5], [0, 2],
                       [0.5, 0], [0.5, 0.5], [0.5, 2],
                       [1, 0], [1, 0.5], [1, 2]]).transpose() - 0.5
        x = np.array([[-0.25, 1.]]).transpose()
    elif index == 2:
        '''inside max_dist = 0.5
        equidistant grid in x-direction only
        '''
        xc = np.array([[0, 0], [0, 0.25], [0, 1], [0, 2],
                       [0.5, 0], [0.5, 0.25], [0.5, 1], [0.5, 2],
                       [1, 0], [1, 0.25], [1, 1], [1, 2]]).transpose() - 0.25
        x = np.array([[-0.25, 1.]]).transpose()
    elif index == 3:
        '''outside max_dist = 0.5'''
        xc = np.array([[-4500, -4000], [-2250, -4000], [0, -4000], [2250, -4000],
                       [-4500, 0], [-2250, 0], [0, 0], [2250, 0],
                       [-4500, 4500], [-2250, 4500], [0, 4500], [2250, 4500],
                       [-4500, 9000], [-2250, 9000], [0, 9000], [2250, 9000]]).transpose()
        x = np.array([[-8000, 8000]]).transpose()
    else:
        '''inside max_dist = 0.5'''
        xc = np.array([[-4500, -4000], [-2250, -4000], [0, -4000], [2250, -4000],
                       [-4500, 0], [-2250, 0], [0, 0], [2250, 0],
                       [-4500, 4500], [-2250, 4500], [0, 4500], [2250, 4500],
                       [-4500, 9000], [-2250, 9000], [0, 9000], [2250, 9000]]).transpose()
        x = np.array([[-3179, 4440]]).transpose()

    return xc, x


def calcDist2D(xc, x, max_dist=0.5):
    pp = xc.max(axis=1) - xc.min(axis=1)
    t = np.diag(1 / pp)
    xc_t = t @ xc
    x_t = t @ x
    dist = x_t - xc_t
    r = np.linalg.norm(dist, axis=0)

    min_dist = np.min(r)

    real_min_dist = np.linalg.inv(t) * min_dist
    real_max_dist = np.linalg.inv(t) * max_dist

    print('Allowed Distance in x: {:.2f}, y: {:.2f}'.format(real_max_dist[0, 0], real_max_dist[1, 1]))
    print('Minimal Distance in x: {:.2f}, y: {:.2f}'.format(real_min_dist[0, 0], real_min_dist[1, 1]))

    ''' check distance threshold'''
    if min_dist > max_dist:
        print("x is located outside of confidence region. {:.2f} > {:.2f}".format(min_dist, max_dist))
    return real_max_dist, real_min_dist


def plot2D(xc, x, max_dist=0.5):

    real_max_dist, real_min_dist = calcDist2D(xc, x, max_dist=max_dist)

    plt.scatter(xc[0], xc[1], c='b')
    plt.scatter(x[0], x[1], c='r')

    for index in range(xc.shape[1]):
        ellipse = Ellipse(xy=(xc[0, index], xc[1, index]), width=2 * real_max_dist[0, 0], height=2 * real_max_dist[1, 1],
                          fill=False)
        ax = plt.gca()
        ax.add_patch(ellipse)
    plt.grid()
    plt.show()


if __name__ == '__main__':
    for _index in range(0, 5):
        _xc, _x = getDataSet2D(_index)
        # _real_max_dist = calcDist2D(_xc, _x)
        plot2D(_xc, _x)
