# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision$
$LastChangedDate::                           $

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import unittest
import numpy as np
import numpy.testing as npt
import lcm.math.rbf.kernels as kernels


class TestKernel(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ref_data = {
            'kernelLS': dict(KernelCLS=kernels.KernelLS, ID=0, params=[],
                             eval=[[1., 1.5, 2., 2.5], [2., 2.5, 3., 3.5]]),
            'KernelCS': dict(KernelCLS=kernels.KernelCS, ID=1, params=[],
                             eval=[[1., 3.375, 8., 15.625], [8., 15.625, 27., 42.875]]),
            'KernelTPS': dict(KernelCLS=kernels.KernelTPS, ID=2, params=[],
                              eval=[[0., 0.912, 2.772, 5.726], [2.772, 5.726, 9.887, 15.346]]),
            'KernelMQ': dict(KernelCLS=kernels.KernelMQ, ID=3, params=[16, 2],
                             eval=[[66049., 332929., 1050625., 2563201.], [1050625., 2563201., 5313025., 9840769.]]),
            'KernelGAUSS': dict(KernelCLS=kernels.KernelGAUSS, ID=4, params=[13],
                                eval=[[0, 0, 0, 0], [0, 0, 0, 0]])
        }

    def test_fromID(self):
        for key, data in self.ref_data.items():
            with self.subTest(msg=key):
                kernel = kernels.AbstractKernel.fromID(kernel_id=data['ID'], kernel_params=data['params'])

                self.assertIsInstance(kernel, data['KernelCLS'])
                self.assertEqual(kernel.id, data['ID'])
                npt.assert_almost_equal(kernel.params, data['params'], decimal=3)

    @unittest.skip
    def test_id(self):
        self.fail()

    @unittest.skip
    def test_params(self):
        self.fail()

    def test_eval(self):
        for key, data in self.ref_data.items():
            with self.subTest(msg=key):
                KernelCLS = data['KernelCLS']
                kernel = KernelCLS(*data['params'])
                result = kernel.eval(np.array([np.arange(1, 3, 0.5), np.arange(2, 4, 0.5)]))

                npt.assert_array_almost_equal(result, np.array(data['eval']), decimal=3)

    @unittest.skip
    def test_dkernel_dradius_dividedby_radius(self):
        self.fail()

    @unittest.skip
    def test_to_msgpack(self):
        self.fail()
