# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 1024 $
$LastChangedDate:: 2020-06-25 18:45:41 +0200#$

This file is part of SyMSpace.

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import unittest
import numpy.testing as npt
import numpy as np
from lcm.tdata_interface.tdata_testing import TDataTesting
from lcm.tdata_interface.pytnumeric_interface import PyTNumericInterface


class TestPyTNumericInterface(unittest.TestCase):

    @unittest.skipIf(PyTNumericInterface.getInterpreter() == 'python', 'needs jep interpreter to run')
    def test_convert_J2Py(self):

        jobj = TDataTesting.getComplexTNumericVector()
        pyobj = PyTNumericInterface.convert_J2Py(jobj)
        npt.assert_array_equal(pyobj, np.atleast_2d(TDataTesting.getComplexNumpyVector()))

        jobj = TDataTesting.getTNumericMatrix()
        pyobj = PyTNumericInterface.convert_J2Py(jobj)
        npt.assert_array_equal(pyobj, TDataTesting.getNumpyMatrix())

    @unittest.skipIf(PyTNumericInterface.getInterpreter() == 'python', 'needs jep interpreter to run')
    def test_convert_Py2J(self):
        pyobj = TDataTesting.getComplexNumpyVector()
        jobj = PyTNumericInterface.convert_Py2J(pyobj)
        pyobj2 = PyTNumericInterface.convert_J2Py(jobj)
        npt.assert_array_equal(np.atleast_2d(pyobj), pyobj2)

        pyobj = TDataTesting.getNumpyMatrix()
        jobj = PyTNumericInterface.convert_Py2J(pyobj)
        pyobj2 = PyTNumericInterface.convert_J2Py(jobj)
        npt.assert_array_equal(pyobj, pyobj2)


if __name__ == "__main__":
    from lcm.utils.testing import runUnittest

    runUnittest(PyTNumericInterface.getInterpreter())
