/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.center.views.tree;

import java.util.ArrayList;

import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

/**
 * 
 * @author Markus Priglinger
 *
 */
public class ProTreeTableIndexViewManager {

	public static final String RepositoryGUID = "ProTreeTableIndexViewManager guid";

	private ProTreeTableHeaderTreeRenderer _treeRenderer = null;
	private ProTreeTableHeaderSelectedIdxRenderer _selectedIdxRenderer = null;
	private ArrayList<IndexRendererPair> _createdIdxRendererList = null;
	private ArrayList<IndexRendererPair> _compareIdxRendererList = null;
	private ArrayList<IndexRendererPair> _visibleIdxRendererList = null;

	private ProTreeTable _table = null;
	private ProParameterModel _tableModel = null;

	public ProTreeTableIndexViewManager(ProTreeTable table) {

		this._table = table;
		this._treeRenderer = new ProTreeTableHeaderTreeRenderer(this._table,
				true);
		this._selectedIdxRenderer = new ProTreeTableHeaderSelectedIdxRenderer(
				this._table);
		this._createdIdxRendererList = new ArrayList<IndexRendererPair>();
		this._compareIdxRendererList = new ArrayList<IndexRendererPair>();
		this._visibleIdxRendererList = new ArrayList<IndexRendererPair>();

		IndexRendererPair treeIRP = new IndexRendererPair(-1,
				this._treeRenderer);
		this._createdIdxRendererList.add(treeIRP);
		this._compareIdxRendererList.add(treeIRP);

		IndexRendererPair selectedIRP = new IndexRendererPair(0,
				this._selectedIdxRenderer);
		this._createdIdxRendererList.add(selectedIRP);
		this._compareIdxRendererList.add(selectedIRP);

		this._visibleIdxRendererList.add(this._compareIdxRendererList.get(0));

		// remove old manager if it exists
		// Repository.getInstance()
		// .removeEntry(ProTreeTableIndexViewManager.RepositoryGUID);

		// // add to repository
		// try {
		// Repository.getInstance().addEntry(
		// ProTreeTableIndexViewManager.RepositoryGUID, this);
		// } catch (RepositoryException e) {
		// e.printStackTrace();
		// }
	}

	public void addIndexView(int idx) {

		IndexRendererPair irp = this.searchIRPinCreated(idx);
		if (irp == null) {
			ProTreeTableHeaderIdxViewRenderer renderer = new ProTreeTableHeaderIdxViewRenderer(
					this._table, "IDX: " + Integer.toString(idx + 1));
			irp = new IndexRendererPair(idx, renderer);
			this._createdIdxRendererList.add(irp);
			this._compareIdxRendererList.add(irp);
		} else {
			// irp exists, check if it is visible. if not found, add it
			if (this.searchIRPinVisible(idx) == null) {
				this._compareIdxRendererList.add(irp);
			}
		}
		this.updateTable();
	}

	public void removeIndexView(int idx) {

		for (int i = 0; i < this._compareIdxRendererList.size(); i++) {
			if (this._compareIdxRendererList.get(i).getIndex() == idx) {
				this._compareIdxRendererList.remove(i);
				this.updateTable();
				return;
			}
		}
	}

	public void removeIndexView(ProTreeTableAbstractHeaderRenderer renderer) {

		for (int i = 0; i < this._compareIdxRendererList.size(); i++) {
			if (this._compareIdxRendererList.get(i).getRenderer() == renderer) {
				this._compareIdxRendererList.remove(i);
				this.updateTable();
				return;
			}
		}
	}

	public int getIndexForColumnPosition(int columnPosition) {

		IndexRendererPair irp = this._visibleIdxRendererList
				.get(columnPosition);
		return irp.getIndex();
	}

	protected IndexRendererPair searchIRPinCreated(int idx) {

		for (int i = 0; i < this._createdIdxRendererList.size(); i++) {
			if (this._createdIdxRendererList.get(i).getIndex() == idx) {
				if (this._createdIdxRendererList.get(i).getRenderer().getClass()
						.equals(ProTreeTableHeaderIdxViewRenderer.class)) {
					return this._createdIdxRendererList.get(i);
				}
			}
		}
		return null;
	}

	protected IndexRendererPair searchIRPinVisible(int idx) {

		for (int i = 0; i < this._compareIdxRendererList.size(); i++) {
			if (this._compareIdxRendererList.get(i).getIndex() == idx) {
				if (this._compareIdxRendererList.get(i).getRenderer().getClass()
						.equals(ProTreeTableHeaderIdxViewRenderer.class)) {
					return this._compareIdxRendererList.get(i);
				}
			}
		}
		return null;
	}

	protected IndexRendererPair searchIRPinVisible(
			ProTreeTableAbstractHeaderRenderer renderer) {

		for (int i = 0; i < this._compareIdxRendererList.size(); i++) {
			if (this._compareIdxRendererList.get(i).getRenderer() == renderer) {
				return this._compareIdxRendererList.get(i);
			}
		}
		return null;
	}

	public void enableActiveIndexView(boolean enable) {

		this._tableModel = this._table.getProParameterModel();

		if (enable) {
			this._visibleIdxRendererList = this._compareIdxRendererList;
		} else {
			this._visibleIdxRendererList = new ArrayList<IndexRendererPair>();
			this._visibleIdxRendererList
					.add(this._compareIdxRendererList.get(0));
		}
		this.updateTable();
	}

	private void updateTable() {

		this._tableModel.setColumnStructure(this._visibleIdxRendererList);
		this._table.getModelAdapter().fireTableStructureChanged();
		this.installHeaderRenderer();
	}

	public void setSelectedIndex(int selectedIndex) {

		IndexRendererPair selectedIndexIRP = this._createdIdxRendererList
				.get(1);
		selectedIndexIRP.setIndex(selectedIndex);
		this._selectedIdxRenderer.setSelectedIndex(selectedIndex);
	}

	// FIXME KW
	// public void indexCountChanged() {
	//
	// Project theProject = GUIManager.getInstance().getProject();
	// if (theProject == null) {
	// return;
	// }
	//
	// // maybe index selected via gui has been removed, get index from project
	// int currentIndex = theProject.getIndex();
	// this.setSelectedIndex(currentIndex);
	// }

	public void installHeaderRenderer() {

		if (this._tableModel == null) {
			this._tableModel = this._table.getProParameterModel();
		}
		int numOfColumns = this._tableModel.getColumnCount();
		TableColumnModel tcModel = this._table.getTableHeader()
				.getColumnModel();

		for (int i = 0; i < numOfColumns; i++) {

			TableColumn tc = tcModel.getColumn(i);
			ProTreeTableAbstractHeaderRenderer renderer = (this._compareIdxRendererList
					.get(i).getRenderer());
			tc.setHeaderRenderer(renderer);
			if (renderer.getClass()
					.equals(ProTreeTableHeaderTreeRenderer.class)) {
				renderer.setToolTipText(
						"Click to hide or show the parameter values");
				tc.setMinWidth(180);
			} else if (renderer.getClass()
					.equals(ProTreeTableHeaderSelectedIdxRenderer.class)) {
				renderer.setToolTipText("Click to add more index views");
				tc.setMinWidth(150);
			} else if (renderer.getClass()
					.equals(ProTreeTableHeaderIdxViewRenderer.class)) {
				renderer.setToolTipText("Index number: "
						+ (this._compareIdxRendererList.get(i).getIndex() + 1));
			}
		}
	}

	public ArrayList<IndexRendererPair> getVisibleColumns() {

		return this._visibleIdxRendererList;
	}
}
