/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.gui;

import java.util.LinkedList;

import javax.swing.JFrame;

/**
 * Base class of GUI's
 *
 * @author Werner Koppelsttter
 */
public class Gui extends JFrame {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Gui instances */
	private static LinkedList<Gui> guis = new LinkedList<Gui>();

	/**
	 * Create GUI
	 * 
	 * @param title
	 *            Title of frame
	 */
	public Gui(String title) {
		super(title);
		Gui.guis.add(this);
	}

	/**
	 * Close GUI. Only if more than on exist
	 */
	public void close() {
		if (Gui.guis.size() > 1) {
			this.setVisible(false);
			this.dispose();
			Gui.guis.remove(this);
		}
	}

	/**
	 * Exit GUI. If more than on exist the close gui
	 */
	public void exit() {
		if (Gui.guis.size() == 1) {
			System.exit(0);
		} else {
			this.close();
		}
	}

}
