/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.optimizer.utils;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.SimpleDateFormat;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SpinnerNumberModel;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import at.lcm.bu21.optim.core.project.ComputeProject;
import at.lcm.symspace.optimizer.icons.opt.OptIcons;

/**
 * Dialog to add stop-contition (savepoint, time)
 * 
 * @author Werner Koppelsttter
 */
public class DialogStopCondition extends JDialog {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Date formatter */
	private SimpleDateFormat sdf = new SimpleDateFormat("H:mm d-MMMMM-YYYY");

	/** Enable savepoint condition */
	JCheckBox chckbxSavepoints;

	/** Number of new savepoints */
	JSpinner spinnerSavepoint;

	/** Stop at savepoint */
	JTextField txtStopSavepoints;

	/** Stop at time */
	JTextField txtStopTime;

	/** Enable time condition */
	JCheckBox chckbxTime;

	/** Time to run */
	JSpinner spinnerTime;

	/** Compute project control */
	ComputeProject compute;

	/**
	 * Show dialog for stop-condition
	 * 
	 * @param compute
	 *            Compute project control
	 * @param parent
	 *            Parent frame
	 */
	public DialogStopCondition(ComputeProject compute, Component parent) {
		this.compute = compute;
		this.setIconImages(OptIcons.getIcons());

		this.setTitle("Stop Condition");
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.getContentPane().setLayout(new BorderLayout());
		this.setModal(true);

		// savepoints
		JPanel panelContition = new JPanel();
		this.getContentPane().add(panelContition, BorderLayout.CENTER);

		this.chckbxSavepoints = new JCheckBox("Stop after new Savepoints");
		this.chckbxSavepoints.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent event) {
				DialogStopCondition.this.spinnerSavepoint.setEnabled(
						DialogStopCondition.this.chckbxSavepoints.isSelected());
			}
		});

		this.spinnerSavepoint = new JSpinner();
		this.spinnerSavepoint.setModel(new SpinnerNumberModel(new Integer(10),
				new Integer(1), null, new Integer(1)));
		this.spinnerSavepoint.setEnabled(false);
		this.spinnerSavepoint.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				DialogStopCondition.this.refresh();
			}
		});

		this.txtStopSavepoints = new JTextField(10);
		this.txtStopSavepoints.setEnabled(false);

		// time
		this.chckbxTime = new JCheckBox("Stop after Time (hour)");
		this.chckbxTime.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				DialogStopCondition.this.spinnerTime.setEnabled(
						DialogStopCondition.this.chckbxTime.isSelected());
			}
		});

		this.spinnerTime = new JSpinner();
		this.spinnerTime.setModel(new SpinnerNumberModel(new Double(10),
				new Double(0), null, new Double(1)));
		this.spinnerTime.setEnabled(false);
		this.spinnerTime.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				DialogStopCondition.this.refresh();
			}
		});

		this.txtStopTime = new JTextField(15);
		this.txtStopTime.setEnabled(false);

		GroupLayout gl_panelContition = new GroupLayout(panelContition);
		gl_panelContition.setHorizontalGroup(gl_panelContition
				.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_panelContition.createSequentialGroup().addGap(20)
						.addGroup(gl_panelContition
								.createParallelGroup(Alignment.LEADING, false)
								.addComponent(this.chckbxTime)
								.addComponent(this.chckbxSavepoints)
								.addGroup(gl_panelContition
										.createSequentialGroup().addGap(20)
										.addGroup(gl_panelContition
												.createParallelGroup(
														Alignment.LEADING,
														false)
												.addComponent(
														this.spinnerSavepoint,
														GroupLayout.DEFAULT_SIZE,
														100, Short.MAX_VALUE)
												.addComponent(
														this.txtStopSavepoints)
												.addComponent(this.spinnerTime)
												.addComponent(
														this.txtStopTime))))));
		gl_panelContition
				.setVerticalGroup(
						gl_panelContition.createParallelGroup(Alignment.LEADING)
								.addGroup(gl_panelContition
										.createSequentialGroup().addGap(20)
										.addComponent(this.chckbxSavepoints)
										.addGap(4)
										.addComponent(this.spinnerSavepoint,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addGap(4)
										.addComponent(this.txtStopSavepoints,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addGap(20)
										.addComponent(this.chckbxTime).addGap(4)
										.addComponent(this.spinnerTime,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addGap(4)
										.addComponent(this.txtStopTime,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addGap(20).addPreferredGap(
												ComponentPlacement.UNRELATED)));
		panelContition.setLayout(gl_panelContition);

		JPanel panelButton = new JPanel();
		panelButton.setBorder(new EmptyBorder(5, 5, 5, 5));
		this.getContentPane().add(panelButton, BorderLayout.SOUTH);
		panelButton.setLayout(new BorderLayout(0, 0));

		JButton btnApply = new JButton("Apply");
		btnApply.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				int sp;
				long time;

				if (DialogStopCondition.this.compute != null) {
					// savepoints
					if (DialogStopCondition.this.chckbxSavepoints
							.isSelected()) {
						sp = DialogStopCondition.this.compute.getProblem()
								.getNumRealSavePoints();
						sp += (Integer) DialogStopCondition.this.spinnerSavepoint
								.getValue();
					} else {
						sp = Integer.MAX_VALUE;
					}
					DialogStopCondition.this.compute
							.setStopConditionSavepoint(sp);

					// time
					if (DialogStopCondition.this.chckbxTime.isSelected()) {
						time = System.currentTimeMillis();
						time += 60 * 60 * 1000
								* ((Double) DialogStopCondition.this.spinnerTime
										.getValue());
					} else {
						time = Long.MAX_VALUE;
					}
					DialogStopCondition.this.compute.setStopConditionTime(time);
				}
				DialogStopCondition.this.setVisible(false);
				DialogStopCondition.this.dispose();
			}
		});
		panelButton.add(btnApply);

		this.setSize(220, 300);
		this.setLocationRelativeTo(parent);

		this.refresh();
		this.setVisible(true);

	}

	/**
	 * Refresh dialog
	 */
	private void refresh() {
		int sp;
		long time;

		if (DialogStopCondition.this.compute != null) {
			sp = this.compute.getProblem().getNumRealSavePoints();
			sp += (Integer) DialogStopCondition.this.spinnerSavepoint
					.getValue();
			this.txtStopSavepoints.setText(Integer.toString(sp));

			time = System.currentTimeMillis();
			time += 60 * 60 * 1000
					* ((Double) DialogStopCondition.this.spinnerTime
							.getValue());
			this.txtStopTime.setText(this.sdf.format(time));
		} else {
			this.txtStopSavepoints.setText("");
			this.txtStopTime.setText("");
		}
	}

}
