/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 10156 $
 * $Date: 2021-04-10 17:32:56 +0200 (Sat, 10 Apr 2021) $
 */
package at.lcm.symspace.optimizer.views;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import at.lcm.bu21.magopt.utils.MagOptConfig;
import at.lcm.bu21.optim.core.plot.WebPlotInterface;
import at.lcm.symspace.gui.utils.Storage;
import at.lcm.symspace.gui.utils.Storage.OptimizationDescription;
import at.lcm.symspace.gui.views.MenuBar;
import at.lcm.symspace.optimizer.OptimizerController;
import at.lcm.symspace.optimizer.icons.IconOptimizerToolbar;

public class OptimizerMenuBar extends MenuBar implements ActionListener {

	// Actions
	private static String ACTION_WIZARD = "Wizard";
	private static String ACTION_SAVE = "Save";
	private static String ACTION_CLOSE = "Close";
	private static String ACTION_OPEN_RECENT = "Open recent optimization";
	private static String ACTION_EXIT = "Exit";

	private static String ACTION_REFRESH = "Refresh";
	private static String ACTION_JOBMANAGER = "Open JobManager";
	private static String ACTION_ADD_PLOT = "Add plot";
	private static String ACTION_REMOVE_PLOT = "Remove plot";

	private static String ACTION_RUN = "Run optimization";
	private static String ACTION_FINISH = "Finish optimization";
	private static String ACTION_STOP = "Stop optimization";
	private static String ACTION_INIT_BUFFFER = "INitialize buffer";
	private static String ACTION_CLEAR_BUFFFER = "Clear buffer";
	private static String ACTION_CREATE_EXCEL = "Create excel";
	private static String ACTION_SELECT = "Select optimum";

	private static String ACTION_MAPPING_CREATE_ALL = "Create all mappings";
	private static String ACTION_MAPPING_OPTIMIZE = "Optmimize mapping";
	private static String ACTION_SENSITIVITY = "Select sensitivity";

	private static String ACTION_SETTINGS_PROBLEM = "Problem problem";
	private static String ACTION_SETTINGS_VARIABLES = "Variables settings";
	private static String ACTION_SETTINGS_RESULTS = "Results settings";
	private static String ACTION_SETTINGS_GENERAL = "General settings";
	private static String ACTION_SETTINGS_SOLVER = "Solver settings";

	private static String ACTION_ASSIGN_PROBLEM_PYTHON = "Assign optimization-problem to Python";
	private static String ACTION_ASSIGN_PROJECT_PYTHON = "Assign optimization-project to Python";
	private static String ACTION_ASSIGN_CONTROLLER_PYTHON = "Assign controller to Python";
	private static String ACTION_OPEN_PROJECT = "Open opimization project";
	private static String ACTION_MACRO = "Run macro";

	private static String ACTION_PLOT_BROWSE = "Browse plot";
	private static String ACTION_PLOT_VARIABLES = "Plot variables";
	private static String ACTION_PLOT_OBJECTIVES = "Plot only objectives";
	private static String ACTION_PLOT_CONSTRAINTS = "Plot objectives and constraints";
	private static String ACTION_PLOT_RESULTS = "Plot all results";
	private static String ACTION_PLOT_JSON = "Plot export json";

	// private static String ACTION_PYTHON_ASSIGN_PARAM = "Assign Parameter to
	// Python";
	// private static String ACTION_PYTHON_ASSIGN_DATA = "Assign Data to
	// Python";
	// private static String ACTION_PYTHON_GET_DATA = "Get Data from Python";
	// private static String ACTION_PYTHON_WRITE_DATA = "Write Data to File";
	//
	// private static String ACTION_MATLAB_ASSIGN_PROJECT = "Assign Project to
	// Matlab";
	// private static String ACTION_MATLAB_ASSIGN_PARAM = "Assign Parameter to
	// Matlab";
	// private static String ACTION_MATLAB_ASSIGN_DATA = "Assign Data to
	// Matlab";
	// private static String ACTION_MATLAB_GET_DATA = "Get Data from Matlab";

	// private static String ACTION_HELP = "Help";
	// private static String ACTION_HELP_UPDATE = "Help Update";
	// private static String ACTION_DOCU = "Docu";
	// private static String ACTION_DOCU_UPDATE = "Docu Update";
	private static String ACTION_ABOUT = "About";

	// KeyStrokes
	private static KeyStroke KEYSTROKE_NONE = null;

	private static KeyStroke KEYSTROKE_WIZARD = KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK);

	private static KeyStroke KEYSTROKE_REFRESH = KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0);
	private static KeyStroke KEYSTROKE_JOBMANAGER = KeyStroke.getKeyStroke(KeyEvent.VK_J, InputEvent.CTRL_DOWN_MASK);

	private static KeyStroke KEYSTROKE_MACRO = KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_DOWN_MASK);

	// private static KeyStroke KEYSTROKE_PYTHON_ASSIGN_PROJECT = KeyStroke
	// .getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_DOWN_MASK);
	// private static KeyStroke KEYSTROKE_PYTHON_ASSIGN_OBJECT = KeyStroke
	// .getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_DOWN_MASK);
	// private static KeyStroke KEYSTROKE_PYTHON_ASSIGN_DATA = KeyStroke
	// .getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_DOWN_MASK);
	// private static KeyStroke KEYSTROKE_RESET_ALL = KeyStroke
	// .getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_DOWN_MASK);
	// private static KeyStroke KEYSTROKE_RESET_SELECTED = KeyStroke
	// .getKeyStroke(KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK);
	// private static KeyStroke KEYSTROKE_UPDATE_SELECTED = KeyStroke
	// .getKeyStroke(KeyEvent.VK_U, InputEvent.CTRL_DOWN_MASK);

//	private static KeyStroke KEYSTROKE_HELP = KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0);

	OptimizerController controller;

	Map<JMenuItem, OptimizationDescription> miRecentFiles;

	// Menu -> File
	JMenu mFile;
	JMenuItem miWizard;
	JMenuItem miSave;
	JMenuItem miClose;
	JMenuItem miExit;

	// Menu -> Display
	JMenuItem miRefresh;
	JMenuItem miJobManager;
	JMenuItem miAddPlot;
	JMenuItem miRemovePlot;

	// Menu -> Project
	JMenuItem miRun;
	JMenuItem miFinish;
	JMenuItem miStop;
	JMenuItem miInitBuffer;
	JMenuItem miClearBuffer;
	JMenuItem miCreateExcel;
	JMenuItem miSelect;

	// Menu -> Mapping
	JMenuItem miMappingCreateAll;
	JMenuItem miMappingOptimize;
	JMenuItem miSensitivity;

	// Menu -> Settings
	JMenuItem miSettingProblem;
	JMenuItem miSettingVariable;
	JMenuItem miSettingResults;
	JMenuItem miSettingGeneral;
	JMenuItem miSettingSolver;

	// Menu -> Debug
	JMenu debugMenu;
	JMenuItem miAssignProblem2Pyhton;
	JMenuItem miAssignProject2Pyhton;
	JMenuItem miAssignController2Pyhton;
	JMenuItem miOpenProject;
	JMenuItem miMacro;

	// Menu -> Debug -> Plot
	JMenuItem miPlotBrowse;
	JPopupMenu.Separator restSeperator;
	JCheckBoxMenuItem miPlotVariables;
	JRadioButtonMenuItem miPlotObjectives;
	JRadioButtonMenuItem miPlotConstraints;
	JRadioButtonMenuItem miPlotResults;
	JMenuItem miPlotJson;

	// Menu -> Help
	// FIXME
	// JMenuItem miHelp;
	// JMenuItem miHelpUpdate;
	// JMenuItem miDocu;
	// JMenuItem miDocuUpdate;
	JMenuItem miAbout;

	public OptimizerMenuBar(OptimizerController controller) {
		this.controller = controller;
		this.miRecentFiles = new Hashtable<JMenuItem, OptimizationDescription>();

		this.add(this.createFileMenu());
		this.add(this.createDisplayMenu());
		this.add(this.createProjectMenu());
		this.add(this.createMappingMenu());
		this.add(this.createSettingsMenu());
		this.add(this.createDebugMenu());
		this.add(this.createHelpMenu());
	}

	private JMenu createFileMenu() {
		this.mFile = new JMenu("File");
		this.mFile.setMnemonic(KeyEvent.VK_F);
		this.mFile.addMenuListener(new MenuListener() {

			@Override
			public void menuSelected(MenuEvent e) {
				List<OptimizationDescription> list;
				File file;
				JMenuItem item;
				int k = 1;
				int key;

				// remove items
				for (JMenuItem entry : OptimizerMenuBar.this.miRecentFiles.keySet()) {
					OptimizerMenuBar.this.mFile.remove(entry);
				}
				// add items
				list = new Storage().getRecentOptimizations();
				if (list != null) {
					for (OptimizationDescription optim : list) {
						file = new File(optim.getMopFile());
						if (k < 10) {
							key = KeyEvent.VK_0 + k;
						} else {
							key = -1;
						}
						item = OptimizerMenuBar.this.makeMenuItem(
								k++ + " " + file.getName() + " : " + optim.getArchive() + " : " + optim.getProblem()
										+ " (" + file.getParent() + ")",
								null, OptimizerMenuBar.ACTION_OPEN_RECENT, key, OptimizerMenuBar.KEYSTROKE_NONE,
								file.getAbsolutePath() + " : " + optim.getArchive() + " : " + optim.getProblem(),
								OptimizerMenuBar.this);
						OptimizerMenuBar.this.miRecentFiles.put(item, optim);
						OptimizerMenuBar.this.mFile.insert(item, OptimizerMenuBar.this.mFile.getItemCount() - 2);
						item.setEnabled(file.exists());
					}
				}
			}

			@Override
			public void menuDeselected(MenuEvent e) {
			}

			@Override
			public void menuCanceled(MenuEvent e) {
			}
		});

		this.miWizard = this.mFile.add(this.makeMenuItem("Wizard", IconOptimizerToolbar.WIZARD,
				OptimizerMenuBar.ACTION_WIZARD, KeyEvent.VK_W, OptimizerMenuBar.KEYSTROKE_WIZARD, "New project", this));
		this.miSave = this.mFile.add(this.makeMenuItem("Create savepoint", null, OptimizerMenuBar.ACTION_SAVE,
				KeyEvent.VK_S, OptimizerMenuBar.KEYSTROKE_NONE, "Create savepoint(Save)", this));
		this.miClose = this.mFile.add(this.makeMenuItem("Close", null, OptimizerMenuBar.ACTION_CLOSE, KeyEvent.VK_C,
				OptimizerMenuBar.KEYSTROKE_NONE, "Close project", this));

		this.mFile.addSeparator();

		// recent Files

		this.mFile.addSeparator();

		this.miExit = this.mFile.add(this.makeMenuItem("Exit", null, OptimizerMenuBar.ACTION_EXIT, KeyEvent.VK_X,
				OptimizerMenuBar.KEYSTROKE_NONE, "Exit", this));

		return this.mFile;
	}

	private JMenu createDisplayMenu() {
		JMenu menu;

		menu = new JMenu("Display");
		menu.setMnemonic(KeyEvent.VK_D);

		this.miRefresh = menu.add(this.makeMenuItem("Refresh", IconOptimizerToolbar.REFRESH,
				OptimizerMenuBar.ACTION_REFRESH, KeyEvent.VK_R, OptimizerMenuBar.KEYSTROKE_REFRESH, "Refresh", this));
		this.miJobManager = menu.add(
				this.makeMenuItem("JobManager", IconOptimizerToolbar.JOBMANAGER, OptimizerMenuBar.ACTION_JOBMANAGER,
						KeyEvent.VK_J, OptimizerMenuBar.KEYSTROKE_JOBMANAGER, "Open JobManager", this));

		menu.addSeparator();

		this.miAddPlot = menu
				.add(this.makeMenuItem("Add plot", IconOptimizerToolbar.ADD_PLOT, OptimizerMenuBar.ACTION_ADD_PLOT,
						KeyEvent.VK_A, OptimizerMenuBar.KEYSTROKE_NONE, "Add user plot", this));
		this.miRemovePlot = menu.add(
				this.makeMenuItem("Remove plot", IconOptimizerToolbar.REMOVE_PLOT, OptimizerMenuBar.ACTION_REMOVE_PLOT,
						KeyEvent.VK_M, OptimizerMenuBar.KEYSTROKE_NONE, "Remove user plot", this));

		return menu;
	}

	private JMenu createProjectMenu() {
		JMenu menu;

		menu = new JMenu("Project");
		menu.setMnemonic(KeyEvent.VK_P);

		this.miRun = menu.add(this.makeMenuItem("Run", IconOptimizerToolbar.RUN, OptimizerMenuBar.ACTION_RUN,
				KeyEvent.VK_R, OptimizerMenuBar.KEYSTROKE_NONE, "Run optimization", this));
		this.miFinish = menu
				.add(this.makeMenuItem("Finish", IconOptimizerToolbar.FINISH, OptimizerMenuBar.ACTION_FINISH,
						KeyEvent.VK_F, OptimizerMenuBar.KEYSTROKE_NONE, "Finish optimization", this));
		this.miStop = menu.add(this.makeMenuItem("Stop", IconOptimizerToolbar.STOP, OptimizerMenuBar.ACTION_STOP,
				KeyEvent.VK_S, OptimizerMenuBar.KEYSTROKE_NONE, "Stop optmization", this));

		menu.addSeparator();

		this.miInitBuffer = menu.add(
				this.makeMenuItem("Init buffer", IconOptimizerToolbar.INIT_BUFFER, OptimizerMenuBar.ACTION_INIT_BUFFFER,
						KeyEvent.VK_I, OptimizerMenuBar.KEYSTROKE_NONE, "Initialize buffer", this));
		this.miClearBuffer = menu.add(this.makeMenuItem("Clear buffer", IconOptimizerToolbar.CLEAR_BUFFER,
				OptimizerMenuBar.ACTION_CLEAR_BUFFFER, KeyEvent.VK_C, OptimizerMenuBar.KEYSTROKE_NONE, "Clear buffer",
				this));

		menu.addSeparator();

		this.miCreateExcel = menu.add(this.makeMenuItem("Write pareto-front to excel", IconOptimizerToolbar.EXCEL,
				OptimizerMenuBar.ACTION_CREATE_EXCEL, KeyEvent.VK_E, OptimizerMenuBar.KEYSTROKE_NONE,
				"Write pareto-front to excel", this));
		this.miSelect = menu
				.add(this.makeMenuItem("Select optimum", IconOptimizerToolbar.SELECT, OptimizerMenuBar.ACTION_SELECT,
						KeyEvent.VK_O, OptimizerMenuBar.KEYSTROKE_NONE, "Select optimum", this));

		return menu;
	}

	private JMenu createMappingMenu() {
		JMenu menu;

		menu = new JMenu("Mapping");
		menu.setMnemonic(KeyEvent.VK_M);

		this.miMappingCreateAll = menu.add(this.makeMenuItem("Create all", IconOptimizerToolbar.MAPPING_CREATE_ALL,
				OptimizerMenuBar.ACTION_MAPPING_CREATE_ALL, KeyEvent.VK_A, OptimizerMenuBar.KEYSTROKE_NONE,
				"Create all mappings", this));
		this.miMappingOptimize = menu.add(this.makeMenuItem("Optimize", IconOptimizerToolbar.MAPPING_OPTIMIZE,
				OptimizerMenuBar.ACTION_MAPPING_OPTIMIZE, KeyEvent.VK_C, OptimizerMenuBar.KEYSTROKE_NONE,
				"Optimize mapping", this));

		menu.addSeparator();

		this.miSensitivity = menu.add(this.makeMenuItem("Select sensitivity", IconOptimizerToolbar.SENSITIVITY_SELECT,
				OptimizerMenuBar.ACTION_SENSITIVITY, KeyEvent.VK_G, OptimizerMenuBar.KEYSTROKE_NONE,
				"Select sensitivity", this));

		return menu;
	}

	private JMenu createSettingsMenu() {
		JMenu menu;

		menu = new JMenu("Settings");
		menu.setMnemonic(KeyEvent.VK_S);

		this.miSettingProblem = menu.add(this.makeMenuItem("Problem", null, OptimizerMenuBar.ACTION_SETTINGS_PROBLEM,
				KeyEvent.VK_P, OptimizerMenuBar.KEYSTROKE_NONE, "Problem settings", this));
		this.miSettingVariable = menu
				.add(this.makeMenuItem("Variables", null, OptimizerMenuBar.ACTION_SETTINGS_VARIABLES, KeyEvent.VK_V,
						OptimizerMenuBar.KEYSTROKE_NONE, "Variables settings", this));
		this.miSettingResults = menu.add(this.makeMenuItem("Results", null, OptimizerMenuBar.ACTION_SETTINGS_RESULTS,
				KeyEvent.VK_R, OptimizerMenuBar.KEYSTROKE_NONE, "Results settings", this));
		this.miSettingGeneral = menu.add(this.makeMenuItem("General", null, OptimizerMenuBar.ACTION_SETTINGS_GENERAL,
				KeyEvent.VK_G, OptimizerMenuBar.KEYSTROKE_NONE, "General settings", this));
		this.miSettingSolver = menu.add(this.makeMenuItem("Solver", null, OptimizerMenuBar.ACTION_SETTINGS_SOLVER,
				KeyEvent.VK_S, OptimizerMenuBar.KEYSTROKE_NONE, "Solver settings", this));

		return menu;
	}

	private JMenu createDebugMenu() {

		this.debugMenu = new JMenu("Debug");
		this.debugMenu.setMnemonic(KeyEvent.VK_G);

		this.miAssignProblem2Pyhton = this.debugMenu
				.add(this.makeMenuItem("Assign problem to python", null, OptimizerMenuBar.ACTION_ASSIGN_PROBLEM_PYTHON,
						KeyEvent.VK_B, OptimizerMenuBar.KEYSTROKE_NONE, "Assign optimization-problem to python", this));
		this.miAssignProject2Pyhton = this.debugMenu
				.add(this.makeMenuItem("Assign project to python", null, OptimizerMenuBar.ACTION_ASSIGN_PROJECT_PYTHON,
						KeyEvent.VK_P, OptimizerMenuBar.KEYSTROKE_NONE, "Assign optimization-project to python", this));
		this.miAssignController2Pyhton = this.debugMenu.add(
				this.makeMenuItem("Assign controller to python", null, OptimizerMenuBar.ACTION_ASSIGN_CONTROLLER_PYTHON,
						KeyEvent.VK_C, OptimizerMenuBar.KEYSTROKE_NONE, "Assign controller to python", this));
		this.miOpenProject = this.debugMenu
				.add(this.makeMenuItem("Open optimization-project", null, OptimizerMenuBar.ACTION_OPEN_PROJECT,
						KeyEvent.VK_O, OptimizerMenuBar.KEYSTROKE_NONE, "Open optimization-project", this));

		this.debugMenu.addSeparator();

		this.miMacro = this.debugMenu.add(this.makeMenuItem("Run Macro", null, OptimizerMenuBar.ACTION_MACRO,
				KeyEvent.VK_M, OptimizerMenuBar.KEYSTROKE_MACRO, "Run Macro", this));

		this.debugMenu.addSeparator();

		this.debugMenu.add(this.createDebugPlotMenu());

		return this.debugMenu;
	}

	private JMenu createDebugPlotMenu() {
		JMenu menu;

		menu = new JMenu("Plot");
		menu.setMnemonic(KeyEvent.VK_T);

		this.miPlotBrowse = menu.add(this.makeMenuItem("Open webplot", null, OptimizerMenuBar.ACTION_PLOT_BROWSE,
				KeyEvent.VK_B, OptimizerMenuBar.KEYSTROKE_NONE, "Browse plot", this));

//		menu.addSeparator();
		this.restSeperator = new JPopupMenu.Separator();
		menu.add(this.restSeperator);

		this.miPlotVariables = this.makeCheckBoxMenuItem("Variales", null, OptimizerMenuBar.ACTION_PLOT_VARIABLES,
				KeyEvent.VK_R, OptimizerMenuBar.KEYSTROKE_NONE, "Variables", this);
		menu.add(this.miPlotVariables);

		menu.addSeparator();

		ButtonGroup group = new ButtonGroup();
		this.miPlotObjectives = this.makeRadioButtonMenuItem("Objectives", null,
				OptimizerMenuBar.ACTION_PLOT_OBJECTIVES, KeyEvent.VK_O, OptimizerMenuBar.KEYSTROKE_NONE, "Objectives",
				this);
		menu.add(this.miPlotObjectives);
		group.add(this.miPlotObjectives);
		this.miPlotConstraints = this.makeRadioButtonMenuItem("Objectives & Constraints", null,
				OptimizerMenuBar.ACTION_PLOT_CONSTRAINTS, KeyEvent.VK_C, OptimizerMenuBar.KEYSTROKE_NONE,
				"Objectives & Contraints", this);
		menu.add(this.miPlotConstraints);
		group.add(this.miPlotConstraints);
		this.miPlotResults = this.makeRadioButtonMenuItem("Results", null, OptimizerMenuBar.ACTION_PLOT_RESULTS,
				KeyEvent.VK_R, OptimizerMenuBar.KEYSTROKE_NONE, "Results", this);
		menu.add(this.miPlotResults);
		group.add(this.miPlotResults);

		menu.addSeparator();

		this.miPlotJson = menu.add(this.makeMenuItem("Export Json", null, OptimizerMenuBar.ACTION_PLOT_JSON,
				KeyEvent.VK_J, OptimizerMenuBar.KEYSTROKE_NONE, "Export Json", this));

		menu.addMenuListener(new MenuListener() {

			@Override
			public void menuSelected(MenuEvent e) {
//				String str;
//				boolean debug = false;
				boolean enable = OptimizerMenuBar.this.isEnabled();
				boolean problemExist = enable && (OptimizerMenuBar.this.controller.getProblem() != null);

//				str = MagOptConfig.getConfig("CenterDebug");
//				if (str != null) {
//					try {
//						debug = Boolean.parseBoolean(str);
//					} catch (Throwable ex) {
//					}
//				}

				OptimizerMenuBar.this.miPlotBrowse.setVisible(WebPlotInterface.exists());
				OptimizerMenuBar.this.miPlotBrowse.setEnabled(problemExist && WebPlotInterface.isAvailable());
				OptimizerMenuBar.this.restSeperator.setVisible(WebPlotInterface.exists());
			}

			@Override
			public void menuDeselected(MenuEvent e) {
			}

			@Override
			public void menuCanceled(MenuEvent e) {
			}
		});

		return menu;
	}

	// private JMenu createDebugPythonMenu() {
	// JMenu menu;
	//
	// menu = new JMenu("Python");
	// menu.setMnemonic(KeyEvent.VK_P);
	//
	// this.miPyAssignProject = menu.add(
	// this.makeMenuItem("Assign Project Handle in Python Console",
	// null, Menu.ACTION_PYTHON_ASSIGN_PROJECT, KeyEvent.VK_P,
	// Menu.KEYSTROKE_PYTHON_ASSIGN_PROJECT,
	// "Assign Project Handle in Python Console"));
	// this.miPyAssignParam = menu.add(this.makeMenuItem(
	// "Assign Selected Object Handle in Python Console", null,
	// Menu.ACTION_PYTHON_ASSIGN_PARAM, KeyEvent.VK_S,
	// Menu.KEYSTROKE_PYTHON_ASSIGN_OBJECT,
	// "Assign Selected Object Handle in Python Console"));
	// this.miPyAssignParam.setDisplayedMnemonicIndex(7);
	// this.miPyAssignData = menu.add(this.makeMenuItem(
	// "Assign Data of Selected Object in Python Console", null,
	// Menu.ACTION_PYTHON_ASSIGN_DATA, KeyEvent.VK_D,
	// Menu.KEYSTROKE_PYTHON_ASSIGN_DATA,
	// "Assign Data of Selected Object in Python Console"));
	// this.miPyGetData = menu.add(this.makeMenuItem(
	// "Get Data from Python Console to Selected Object", null,
	// Menu.ACTION_PYTHON_GET_DATA, KeyEvent.VK_G, Menu.KEYSTROKE_NONE,
	// "Get Data from Python Console to Selected Object"));
	// this.miPyWriteData = menu.add(this.makeMenuItem(
	// "Write Data of Selected Object to Pickle-File", null,
	// Menu.ACTION_PYTHON_WRITE_DATA, KeyEvent.VK_F,
	// Menu.KEYSTROKE_NONE,
	// "Write Data of Selected Object to Pickle-File"));
	//
	// return menu;
	// }

	// private JMenu createDebugMatlabMenu() {
	// JMenu menu;
	//
	// menu = new JMenu("Matlab");
	// menu.setMnemonic(KeyEvent.VK_M);
	//
	// this.miMatlabAssignProject = menu.add(this.makeMenuItem(
	// "Assign Project Handle in Workspace", null,
	// Menu.ACTION_MATLAB_ASSIGN_PROJECT, KeyEvent.VK_P,
	// Menu.KEYSTROKE_NONE, "Assign Project Handle in Workspace"));
	// this.miMatlabAssignParam = menu.add(
	// this.makeMenuItem("Assign Selected Object Handle in Workspace",
	// null, Menu.ACTION_MATLAB_ASSIGN_PARAM, KeyEvent.VK_S,
	// Menu.KEYSTROKE_NONE,
	// "Assign Selected Object Handle in Workspace"));
	// this.miMatlabAssignParam.setDisplayedMnemonicIndex(7);
	// this.miMatlabAssignData = menu.add(
	// this.makeMenuItem("Assign Data of Selected Object in Workspace",
	// null, Menu.ACTION_MATLAB_ASSIGN_DATA, KeyEvent.VK_D,
	// Menu.KEYSTROKE_NONE,
	// "Assign Data of Selected Object in Workspace"));
	// this.miMatlabGetData = menu.add(this.makeMenuItem(
	// "Get Data from Matlab workspace to Selected Object", null,
	// Menu.ACTION_MATLAB_GET_DATA, KeyEvent.VK_G, Menu.KEYSTROKE_NONE,
	// "Get Data from Matlab workspace to Selected Object"));
	//
	// return menu;
	// }

	private JMenu createHelpMenu() {
		JMenu menu;

		menu = new JMenu("Help");
		menu.setMnemonic(KeyEvent.VK_H);

		// this.miHelp = menu.add(this.makeMenuItem("Help", null,
		// Menu.ACTION_HELP,
		// KeyEvent.VK_H, Menu.KEYSTROKE_HELP, "Help"));
		// this.miHelpUpdate = menu.add(
		// this.makeMenuItem("Update Help", null, Menu.ACTION_HELP_UPDATE,
		// KeyEvent.VK_U, Menu.KEYSTROKE_NONE, "Update Help"));
		//
		// menu.addSeparator();
		//
		// this.miDocu = menu
		// .add(this.makeMenuItem("Documentation", null, Menu.ACTION_DOCU,
		// KeyEvent.VK_D, Menu.KEYSTROKE_NONE, "Documetation"));
		// this.miDocuUpdate = menu.add(this.makeMenuItem("Update
		// Documentation",
		// null, Menu.ACTION_DOCU_UPDATE, KeyEvent.VK_T,
		// Menu.KEYSTROKE_NONE, "Update Documetation"));
		//
		// menu.addSeparator();

		this.miAbout = menu.add(this.makeMenuItem("About SyMSpace Optimizer", null, OptimizerMenuBar.ACTION_ABOUT,
				KeyEvent.VK_A, OptimizerMenuBar.KEYSTROKE_NONE, "About SyMSpace Optimizer", this));

		return menu;
	}

	@Override
	public void actionPerformed(ActionEvent e) {

		// File
		if (e.getActionCommand() == OptimizerMenuBar.ACTION_WIZARD) {
			this.controller.mnu_file_wizard();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_SAVE) {
			this.controller.mnu_file_save();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_CLOSE) {
			this.controller.mnu_file_close();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_OPEN_RECENT) {
			OptimizationDescription desc = this.miRecentFiles.get(e.getSource());
			this.controller.mnu_file_wizard(new File(desc.getMopFile()), desc.getArchive(), desc.getProblem());
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_EXIT) {
			this.controller.mnu_file_exit();

			// Display
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_JOBMANAGER) {
			this.controller.mnu_display_jobmanager();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_REFRESH) {
			this.controller.mnu_display_refresh();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_ADD_PLOT) {
			this.controller.mnu_display_add_plot();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_REMOVE_PLOT) {
			this.controller.mnu_display_remove_plot();

			// Project
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_RUN) {
			this.controller.mnu_project_run();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_FINISH) {
			this.controller.mnu_project_finish();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_STOP) {
			this.controller.mnu_project_stop();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_INIT_BUFFFER) {
			this.controller.mnu_project_init_buffer();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_CLEAR_BUFFFER) {
			this.controller.mnu_project_clear_buffer();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_CREATE_EXCEL) {
			this.controller.mnu_project_create_excel();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_SELECT) {
			this.controller.mnu_project_select();

			// Mapping
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_MAPPING_CREATE_ALL) {
			this.controller.mnu_mapping_create_all();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_MAPPING_OPTIMIZE) {
			this.controller.mnu_mapping_optimize();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_SENSITIVITY) {
			this.controller.mnu_mapping_sensitivity();

			// Settings
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_SETTINGS_PROBLEM) {
			this.controller.mnu_settings_problem();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_SETTINGS_VARIABLES) {
			this.controller.mnu_settings_variables();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_SETTINGS_RESULTS) {
			this.controller.mnu_settings_results();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_SETTINGS_GENERAL) {
			this.controller.mnu_settings_general();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_SETTINGS_SOLVER) {
			this.controller.mnu_settings_solver();

			// Debug
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_ASSIGN_PROBLEM_PYTHON) {
			this.controller.mnu_debug_assign_problem_python();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_ASSIGN_PROJECT_PYTHON) {
			this.controller.mnu_debug_assign_project_python();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_ASSIGN_CONTROLLER_PYTHON) {
			this.controller.mnu_debug_assign_controller_python();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_OPEN_PROJECT) {
			this.controller.mnu_debug_assign_open_project();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_MACRO) {
			this.controller.mnu_debug_macro();
			// Debug -> Plot
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_PLOT_BROWSE) {
			this.controller.mnu_plot_browse();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_PLOT_VARIABLES) {
			this.controller.mnu_plot_variables();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_PLOT_OBJECTIVES) {
			this.controller.mnu_plot_objectives();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_PLOT_CONSTRAINTS) {
			this.controller.mnu_plot_constraints();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_PLOT_RESULTS) {
			this.controller.mnu_plot_results();
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_PLOT_JSON) {
			this.controller.mnu_plot_json();
			// Help
		} else if (e.getActionCommand() == OptimizerMenuBar.ACTION_ABOUT) {
			this.controller.mnu_help_about();
		} else {
			System.out.println("Not implemented");
		}

	}

	@Override
	public void refresh() {
		// FIXME
		boolean problemExist;
		boolean debug = false;
		String str;

		problemExist = this.controller.getProblem() != null;

		str = MagOptConfig.getConfig("OptimizerDebug");
		if (str != null) {
			try {
				debug = Boolean.parseBoolean(str);
			} catch (Throwable ex) {
			}
		}

		// FIXME dirty
		// Menu -> File
		this.miWizard.setEnabled(true);
		this.miSave.setEnabled(problemExist);
		this.miClose.setEnabled(problemExist);
		this.miExit.setEnabled(true);

		// Menu -> Display
		this.miRefresh.setEnabled(true);
		this.miJobManager.setEnabled(true);
		this.miAddPlot.setEnabled(problemExist);
		this.miRemovePlot.setEnabled(problemExist && (this.controller.getProblem().listUserPlots().length > 0));

		// Menu -> Project
		this.miRun.setEnabled(this.controller.isRunEnabled());
		this.miFinish.setEnabled(this.controller.isFinishEnabled());
		this.miStop.setEnabled(this.controller.isStopEnabled());

		this.miInitBuffer.setEnabled(problemExist);
		this.miClearBuffer.setEnabled(problemExist);
		this.miCreateExcel.setEnabled(problemExist);
		this.miSelect.setEnabled(problemExist);

		// Menu -> mapping
		this.miMappingCreateAll.setEnabled(this.controller.isCreateMappingEnabled());
		this.miMappingOptimize.setEnabled(this.controller.isRunMappingEnabled());
		this.miSensitivity.setEnabled(this.controller.isSensitivityInitializationEnabled());

		// Menu -> settings
		this.miSettingProblem.setEnabled(problemExist);
		this.miSettingVariable.setEnabled(problemExist);
		this.miSettingResults.setEnabled(problemExist);
		this.miSettingGeneral.setEnabled(problemExist);
		this.miSettingSolver.setEnabled(problemExist);

		// Menu -> debug
		this.debugMenu.setVisible(debug);
		this.miAssignProblem2Pyhton.setEnabled(problemExist);
		this.miAssignProject2Pyhton.setEnabled(problemExist);
		this.miAssignController2Pyhton.setEnabled(true);
		this.miOpenProject.setEnabled(problemExist);
		this.miMacro.setEnabled(true);
		// Menu -> debug -> plot
		this.miPlotVariables.setSelected(this.controller.isPlotVariables());
		this.miPlotObjectives.setSelected(this.controller.isPlotObjectives());
		this.miPlotConstraints.setSelected(this.controller.isPlotConstraints());
		this.miPlotResults.setSelected(this.controller.isPlotResults());
		this.miPlotJson.setEnabled(problemExist);

		// Menu -> Help
		// this.miHelp.setEnabled(this.controller.exist_help());
		// this.miHelpUpdate.setEnabled(true);
		// this.miDocu.setEnabled(this.controller.exist_docu());
		// this.miDocuUpdate.setEnabled(true);
		this.miAbout.setEnabled(true);

	}

}
