/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure;

import java.rmi.RemoteException;

import magopt.structure.database.DBItem;
import magopt.structure.database.MaterialConnection;

/**
 * Interface of material
 * 
 * @author Werner Koppelsttter
 */
public interface IParamMaterial extends IParamContainer, IFunction {

	/**
	 * Get material-class-name
	 * 
	 * @return Material-class name
	 * @throws RemoteException
	 */
	public String getMaterialClassName() throws RemoteException;

	/**
	 * Get material-name
	 * 
	 * @return material-name
	 * @throws RemoteException
	 */
	public String getMaterialName() throws RemoteException;

	/**
	 * Get material-name
	 * 
	 * @return material-name
	 * @throws RemoteException
	 */
	public String getDatasetName() throws RemoteException;

	/**
	 * Get ID of dataset
	 * 
	 * @return ID
	 * @throws RemoteException
	 */
	public long getDatasetID() throws RemoteException;

	/**
	 * Set material of project
	 * 
	 * @param dataset
	 *            Dataset of selected material
	 * @param conn
	 *            Connection to database
	 * @throws RemoteException
	 */
	public void setDatasetID(DBItem dataset, MaterialConnection conn)
			throws RemoteException;

	/**
	 * Set material of project
	 * 
	 * @param datasetID
	 *            ID of dataset
	 * @param conn
	 *            Connection to database
	 * @throws RemoteException
	 */
	public void setDatasetID(long datasetID, MaterialConnection conn)
			throws RemoteException;

	@Override
	public IParamMaterial getClientImpl() throws RemoteException;
}
