/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.client;

import java.rmi.RemoteException;
import java.util.Map;

import at.lcm.bu21.magopt.structure.FormulaType;
import at.lcm.bu21.magopt.structure.IDepend;
import at.lcm.bu21.magopt.structure.IParamFormula;

/**
 * Client adapter for formula interface
 * 
 * @author Werner Koppelsttter
 */
public class CParamFormula extends CParamField implements IParamFormula {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** formula class */
	private IParamFormula param;

	/**
	 * Create client adapter for formula interface
	 * 
	 * @param param
	 *            Formula
	 */
	public CParamFormula(IParamFormula param) {
		super(param);
		this.param = param;
	}

	@Override
	public boolean getUpdateRequired() throws RemoteException {
		return this.param.getUpdateRequired();
	}

	@Override
	public boolean getUpdateRequired(int index) throws RemoteException {
		return this.param.getUpdateRequired(index);
	}

	@Override
	public void setUpdateRequired(boolean update) throws RemoteException {
		this.param.setUpdateRequired(update);
	}

	@Override
	public void setUpdateRequired(int index, boolean update)
			throws RemoteException {
		this.param.setUpdateRequired(index, update);
	}

	@Override
	public FormulaType getFormulaType() throws RemoteException {
		return this.param.getFormulaType();
	}

	@Override
	public void setFormulaType(FormulaType type) throws RemoteException {
		this.param.setFormulaType(type);
	}

	@Override
	public String getExpression() throws RemoteException {
		return this.param.getExpression();
	}

	@Override
	public void setExpression(String exp) throws RemoteException {
		this.param.setExpression(exp);
	}

	@Override
	public boolean isEnable() throws RemoteException {
		return this.param.isEnable();
	}

	@Override
	public IDepend getDepend() throws RemoteException {
		return this.param.getDepend().getClientImpl();
	}

	@Override
	public void reset() throws RemoteException {
		this.param.reset();
	}

	@Override
	public int update(Map<String, Object> args) throws Exception,
			RemoteException {
		return this.param.update(args);
	}

	@Override
	public void refreshDepend(boolean dirty, boolean load)
			throws RemoteException {
		this.param.refreshDepend(dirty, load);
	}

	@Override
	public void refreshDependency() throws RemoteException {
		this.param.refreshDependency();
	}

	@Override
	public CParamFormula getClientImpl() {
		return this;
	}
}
