/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import java.io.File;
import java.util.AbstractList;
import java.util.LinkedList;

import at.lcm.bu21.magopt.structure.Project;

/**
 * Convert all library project and templates-files to the current version
 * 
 * @author Werner Koppelsttter
 */
public class ConvertLibrary {

	/**
	 * Start conversion as application
	 * 
	 * @param args
	 *            Calling arguments
	 */
	public static void main(String[] args) {
		new ConvertLibrary();
	}

	/**
	 * Constructor with convert the library-files to the current version
	 */
	public ConvertLibrary() {
		int i = 1, size, count = 0;
		String path;
		File file;
		LinkedList<File> fileList;
		XMLConverter converter;

		path = Project.getInstallPath();
		if (path != null) {
			file = new File(path);
			if (file.exists() && file.isDirectory()) {
				fileList = new LinkedList<File>();
				this.collectFiles(file, fileList);

				System.out.println("File List");
				size = fileList.size();
				for (File f : fileList) {
					try {
						converter = new XMLConverter(f, Project.getVersion());
						if (converter.compareVersion() != 0) {
							System.out.println("(" + i++ + "/" + size
									+ ") Convert: " + f.getPath());
							converter.convert();
							count++;
						} else {
							System.out.println("(" + i++ + "/" + size
									+ ") Up-to-date: " + f.getPath());
						}
					} catch (Exception ex) {
						System.out.println("ERROR by file: " + f.getPath());
					}
				}
				System.out.println(count + " of " + size + " Converted");
			}
		}
	}

	/**
	 * Collect recursive project and template-files
	 * 
	 * @param root
	 *            search directory
	 * @param fileList
	 *            list with the found files
	 */
	private void collectFiles(File root, AbstractList<File> fileList) {
		File[] list;
		list = root.listFiles();
		for (int k = 0; k < list.length; k++) {
			if (list[k].isDirectory()) {
				this.collectFiles(list[k], fileList);
			} else {
				if (list[k].getName().matches(".+\\.mop")
						|| list[k].getName().matches(".+\\.mopt")
						|| list[k].getName().matches(".+\\.mopc")) {
					fileList.add(list[k]);
				}
			}
		}
	}

}
