/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import at.lcm.bu21.general.utils.XMLNode;

/**
 * Convert to V_0_23.
 * 
 * - Add Report-parameters
 * 
 * @author Werner Koppelsttter
 */
public class Converter_23 {

	/**
	 * Convert the given node
	 * 
	 * @param node
	 *            Data to convert
	 * @return Changed data node
	 */
	static XMLNode convert(XMLNode node) {

		// Convert Event
		Converter_23.convertReport(node);

		return node;
	}

	/**
	 * Add report-parameters
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void convertReport(XMLNode node) {
		XMLNode report, item, prop, entry;

		report = new XMLNode("ReportParameter");
		node.addNode(5, report);

		// Style
		item = new XMLNode("Style");
		report.addNode(item);
		item.setText("MAGOPT");

		// Language
		item = new XMLNode("Language");
		report.addNode(item);
		item.setText("GERMAN");

		// Properties
		item = new XMLNode("Properties");
		report.addNode(item);

		// ProjectMember
		prop = new XMLNode("prop");
		item.addNode(prop);

		entry = new XMLNode("Key");
		prop.addNode(entry);
		entry.setText("ProjectMember");

		entry = new XMLNode("Value");
		prop.addNode(entry);
		entry.setText("MagOpt");

		// ProjectName
		prop = new XMLNode("prop");
		item.addNode(prop);

		entry = new XMLNode("Key");
		prop.addNode(entry);
		entry.setText("ProjectName");

		entry = new XMLNode("Value");
		prop.addNode(entry);
		entry.setText("Project");
	}
}
