/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 10126 $
 * $Date: 2021-03-09 10:12:08 +0100 (Tue, 09 Mar 2021) $
 */
package at.lcm.symspace.utils;

import java.lang.reflect.Method;

import at.lcm.bu21.magopt.structure.Project;

public class WebguiInterface {

	public static Class<?> server = null;

	static {
		try {
			WebguiInterface.server = Class.forName("at.lcm.symspace.rest.RESTServer");
		} catch (ClassNotFoundException e) {
		}

	}

	public static void setProject(Project project) {
		Method m;

		if (WebguiInterface.exists()) {
			try {
				m = WebguiInterface.server.getMethod("setProject", Project.class);
				m.invoke(null, project);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public static void browse() {
		Method m;

		if (WebguiInterface.exists()) {
			try {
				m = WebguiInterface.server.getMethod("browse");
				m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public static void startServer() {
		Method m;

		if (WebguiInterface.exists()) {
			try {
				m = WebguiInterface.server.getMethod("startServer");
				m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public static void stopServer() {
		Method m;

		if (WebguiInterface.exists()) {
			try {
				m = WebguiInterface.server.getMethod("stopServer");
				m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public static boolean isRunning() {
		boolean running = false;
		Method m;

		if (WebguiInterface.exists()) {
			try {
				m = WebguiInterface.server.getMethod("isRunning");
				running = (Boolean) m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}

		return running;
	}

	public static boolean isAvailable() {
		boolean available = false;
		Method m;

		if (WebguiInterface.exists()) {
			try {
				m = WebguiInterface.server.getMethod("isAvailable");
				available = (Boolean) m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}

		return available;
	}

	public static boolean exists() {
		return WebguiInterface.server != null;
	}

}
