/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.event;

/**
 * Connects column indices with column header names.
 * 
 * @author Werner Koppelsttter
 */
public enum EventColumn {
	/** Type of Event */
	TYPE(0, ""),
	/** Date of Event */
	DATE(1, "Date"),
	/** Description of Event */
	DESCRIPTION(2, "Description");

	/** Column number */
	private final int column;
	/** Column name */
	private final String name;

	/**
	 * Create enumeration
	 * 
	 * @param column
	 *            Column number
	 * @param name
	 *            Column name
	 */
	private EventColumn(int column, String name) {
		this.column = column;
		this.name = name;
	}

	/**
	 * Get column number
	 * 
	 * @return Number
	 */
	int getValue() {
		return this.column;
	}

	/**
	 * Get column name
	 * 
	 * @return Name
	 */
	String getName() {
		return this.name;
	}

}
