/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9752 $
 * $Date: 2020-04-11 00:01:35 +0200 (Sat, 11 Apr 2020) $
 */
package magopt.gui.helpers;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import at.lcm.bu21.general.dtypes.editor2.TDataEditor;
import at.lcm.bu21.general.utils.CloneException;
import at.lcm.bu21.magopt.structure.ParamField;
import at.lcm.bu21.magopt.structure.Parameter;

/**
 * Display data-editor
 * 
 * @author Werner Koppelsttter
 */
public class DataEditor extends JDialog {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Data-editor */
	// private Editor editor = null;
	private TDataEditor editor = null;

	/** Parameter with data */
	private Parameter param = null;

	/** Label for parameter identifier */
	private JLabel label = null;
	/** Button to assign changes */
	public JButton apply = null;
	/** Button to close dialog */
	private JButton close = null;

	/**
	 * Open data-editor
	 * 
	 * @param param
	 *            Parameter for edit data
	 */
	public DataEditor(Parameter param) {
		this(param, null);
	}

	/**
	 * The frame from which the dialog is displayed
	 * 
	 * @param param
	 *            Parameter for edit data
	 * @param owner
	 *            Frame from which the dialog is displayed
	 */
	public DataEditor(Parameter param, Frame owner) {
		super(owner);
		JPanel panel;

		this.param = param;
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		this.setLayout(new BorderLayout());
		// this.editor = new Editor();
		// this.add(this.editor.getPanel(), BorderLayout.CENTER);
		//
		// try {
		// this.editor.addData("data", this.param.getData().deepClone());
		// } catch (CloneException e) {
		// }
		try {
			this.editor = new TDataEditor(this.param.getData().deepClone());
			this.add(this.editor.getPanel(), BorderLayout.CENTER);
		} catch (CloneException e) {
		}

		panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		this.add(panel, BorderLayout.SOUTH);

		this.label = new JLabel(this.param.getIdentifier());
		panel.add(this.label);

		panel.add(Box.createHorizontalGlue());

		this.apply = new JButton("Apply");
		this.apply.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					// DataEditor.this.param
					// .setData(DataEditor.this.editor.getTData("data"));
					DataEditor.this.param.setData(DataEditor.this.editor.getData());
					DataEditor.this.dispose();
				} catch (Exception e1) {
					System.out.println(e1.getMessage());
					e1.printStackTrace();
				}
			}
		});
		this.apply.setEnabled(!(this.param.getReadOnly()
				|| ((this.param instanceof ParamField) && ((ParamField) this.param).getLock())));
		panel.add(this.apply);

		this.close = new JButton("Cancel");
		this.close.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				DataEditor.this.dispose();
			}
		});
		panel.add(this.close);

		this.pack();
		this.setVisible(true);

	}
}
