/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.helpers;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.rmi.RemoteException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import magopt.structure.database.DBItem;
import magopt.structure.database.MaterialConnection;
import at.lcm.bu21.magopt.structure.IParamMaterial;

/**
 * Dialog to select for set report-structure
 * 
 * @author Werner Koppelsttter
 */
public class SelectMaterialDialog {

	/** Dialog to select material */
	private JDialog dialog;

	/** JComboBox for material-class */
	private JComboBox materialClass;

	/** JComboBox for material */
	private JComboBox material;

	/** JComboBox for dataset */
	private JComboBox dataset;

	/** Material-parameter of change */
	private IParamMaterial mat;

	/** Connection to database */
	private MaterialConnection conn = null;

	/**
	 * Constructor of dialog to select material
	 * 
	 * @param parent
	 *            Parent of dialog
	 */
	public SelectMaterialDialog(Frame parent) {
		this.init(parent);
		this.dialog.setLocationRelativeTo(parent);
		this.dialog.setSize(new Dimension(400, 200));
	}

	/**
	 * Open dialog with given structure
	 * 
	 * @param material
	 *            Material object
	 * @param conn
	 *            Connection to database
	 * @throws Exception
	 */
	public void show(IParamMaterial material, MaterialConnection conn)
			throws Exception {
		DBItem[] list;

		if (material != null) {
			this.mat = material;
			this.conn = conn;

			// material-class
			list = this.conn.getMaterialClassList();
			for (DBItem item : list) {
				this.materialClass.addItem(item);
			}
			this.materialClass.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					DBItem[] list;

					SelectMaterialDialog.this.material.removeAllItems();
					list = SelectMaterialDialog.this.conn
							.getMaterialList((DBItem) SelectMaterialDialog.this.materialClass
									.getSelectedItem());
					for (DBItem item : list) {
						SelectMaterialDialog.this.material.addItem(item);
					}
				}
			});

			// material
			this.material.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					DBItem[] list;

					SelectMaterialDialog.this.dataset.removeAllItems();
					list = SelectMaterialDialog.this.conn
							.getDatasetList((DBItem) SelectMaterialDialog.this.material
									.getSelectedItem());
					for (DBItem item : list) {
						SelectMaterialDialog.this.dataset.addItem(item);
					}
					if (list.length == 2) {
						SelectMaterialDialog.this.dataset.setSelectedIndex(1);
					}
				}
			});

			// set selected
			try {
				list = this.conn.getMaterialItems(this.mat.getDatasetID());
				this.materialClass.setSelectedItem(list[0]);
				this.material.setSelectedItem(list[1]);
				this.dataset.setSelectedItem(list[2]);
			} catch (RemoteException e1) {
				e1.printStackTrace();
			}

			this.dialog.setVisible(true);
		}

	}

	/**
	 * Initialize dialog
	 * 
	 * @param parent
	 *            Parent frame
	 */
	protected void init(Frame parent) {
		JPanel mainPanel, panel;
		JButton button;
		JLabel label;
		GridBagLayout layout;
		GridBagConstraints c;
		// Font font;

		// create dialog
		this.dialog = new JDialog(parent, "Select material", true);
		this.dialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				SelectMaterialDialog.this.dialog.setVisible(false);
				SelectMaterialDialog.this.conn.close();
				SelectMaterialDialog.this.dialog.dispose();
			}
		});
		this.dialog.setLayout(new BorderLayout());

		mainPanel = new JPanel();
		mainPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		mainPanel.setLayout(new BorderLayout());
		this.dialog.add(mainPanel, BorderLayout.CENTER);

		// // top area
		// label = new JLabel("Selct material");
		// font = label.getFont();
		// font = font.deriveFont(Font.BOLD, font.getSize() * 1.4f);
		// label.setFont(font);
		// mainPanel.add(label, BorderLayout.NORTH);

		// center area
		panel = new JPanel();
		layout = new GridBagLayout();
		c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(5, 5, 5, 5);
		panel.setLayout(layout);
		mainPanel.add(panel, BorderLayout.CENTER);

		label = new JLabel("Material-class:");
		c.gridwidth = 1;
		c.weightx = 0.0;
		layout.setConstraints(label, c);
		panel.add(label);
		this.materialClass = new JComboBox();
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1.0;
		layout.setConstraints(this.materialClass, c);
		panel.add(this.materialClass);

		label = new JLabel("Material:");
		c.gridwidth = 1;
		c.weightx = 0.0;
		layout.setConstraints(label, c);
		panel.add(label);
		this.material = new JComboBox();
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1.0;
		layout.setConstraints(this.material, c);
		panel.add(this.material);

		label = new JLabel("Dataset:");
		c.gridwidth = 1;
		c.weightx = 0.0;
		layout.setConstraints(label, c);
		panel.add(label);
		this.dataset = new JComboBox();
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1.0;
		layout.setConstraints(this.dataset, c);
		panel.add(this.dataset);

		// add buttons
		panel = new JPanel();
		panel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		mainPanel.add(panel, BorderLayout.SOUTH);

		panel.add(Box.createHorizontalGlue());

		button = new JButton("OK");
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SelectMaterialDialog.this.dialog.setVisible(false);
				try {
					SelectMaterialDialog.this.mat.setDatasetID(
							(DBItem) SelectMaterialDialog.this.dataset
									.getSelectedItem(),
							SelectMaterialDialog.this.conn);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
				SelectMaterialDialog.this.conn.close();
				SelectMaterialDialog.this.dialog.dispose();
			}
		});
		panel.add(button);

		panel.add(Box.createHorizontalGlue());

		button = new JButton("Cancel");
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SelectMaterialDialog.this.dialog.setVisible(false);
				SelectMaterialDialog.this.conn.close();
				SelectMaterialDialog.this.dialog.dispose();
			}
		});
		panel.add(button);

		panel.add(Box.createHorizontalGlue());

		this.dialog.pack();
	}

}
