/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JPanel;

import at.lcm.bu21.general.dtypes.TNull;
import at.lcm.bu21.magopt.structure.IParamContainer;
import at.lcm.bu21.magopt.structure.IParamDirectory;
import at.lcm.bu21.magopt.structure.IParamField;
import at.lcm.bu21.magopt.structure.IParamFile;
import at.lcm.bu21.magopt.structure.IParamFormula;
import at.lcm.bu21.magopt.structure.IParamFunction;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.IProject;

/**
 * Displays Flags.
 * 
 * @author Aigner Markus
 * 
 */
public class FlagsView extends ParamBaseScroll {
	private static final long serialVersionUID = 1L;

	private IParameter mParam;

	JCheckBox mModified;
	JCheckBox mReadOnly;
	JCheckBox mDataEmpty;
	JCheckBox mUsed;
	JCheckBox mUpdateRequired;
	JCheckBox mUncertain;
	JCheckBox mIncomplete;
	JCheckBox mError;
	JCheckBox mDataBase;
	JCheckBox mPrint;

	private JPanel mainPanel;

	public FlagsView() {
		this.mainPanel = new JPanel();
		this.mainPanel.setVisible(false);
		this.mainPanel.setLayout(new BoxLayout(this.mainPanel, BoxLayout.Y_AXIS));
		this.mainPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		this.add(this.mainPanel);

		/* flags of class Parameter */
		this.mModified = new JCheckBox("Modified");
		this.mModified.setEnabled(false);
		this.mainPanel.add(this.mModified);
		this.mUncertain = new JCheckBox("Uncertain");
		this.mUncertain.setEnabled(false);
		this.mainPanel.add(this.mUncertain);
		this.mUsed = new JCheckBox("Used");
		this.mUsed.setEnabled(false);
		this.mainPanel.add(this.mUsed);
		this.mReadOnly = new JCheckBox("Read Only");
		this.mReadOnly.setEnabled(false);
		this.mainPanel.add(this.mReadOnly);
		this.mError = new JCheckBox("Error");
		this.mError.setEnabled(false);
		this.mainPanel.add(this.mError);
		this.mPrint = new JCheckBox("Print");
		this.mPrint.setEnabled(false);
		this.mainPanel.add(this.mPrint);
		this.mPrint.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent event) {
				System.out.println("Print" + FlagsView.this.mPrint.isSelected());
				try {
					FlagsView.this.mParam.setPrint(FlagsView.this.mPrint.isSelected());
				} catch (RemoteException ex) {
					System.err.println(ex.getMessage());
					ex.printStackTrace();
				}
				FlagsView.this.update();
			}
		});

		/* flags dependent of Parameter derived class */
		this.mDataEmpty = new JCheckBox("Data Empty");
		this.mDataEmpty.setEnabled(false);
		this.mainPanel.add(this.mDataEmpty);
		this.mUpdateRequired = new JCheckBox("Update Required");
		this.mUpdateRequired.setEnabled(false);
		this.mainPanel.add(this.mUpdateRequired);
		this.mIncomplete = new JCheckBox("Incomplete");
		this.mIncomplete.setEnabled(false);
		this.mainPanel.add(this.mIncomplete);
		this.mDataBase = new JCheckBox("DataBase");
		this.mDataBase.setEnabled(false);
		this.mainPanel.add(this.mDataBase);
	}

	/**
	 * Sets new Parameter.
	 * 
	 * @param param
	 *            - Parameter
	 */
	public void setParameter(IParameter param) {
		this.mParam = param;
		this.mainPanel.setVisible(this.mParam != null);
		this.update();
	}

	@Override
	public void update() {
		this.update(this.mParam);

		try {
			if (this.mParam instanceof IProject) {
				this.update((IProject) this.mParam);
			} else if (this.mParam instanceof IParamContainer) {
				this.update((IParamContainer) this.mParam);
			} else if (this.mParam instanceof IParamField) {
				this.update(this.mParam);
			} else if (this.mParam instanceof IParamFunction) {
				this.update((IParamFunction) this.mParam);
			} else if (this.mParam instanceof IParamFormula) {
				this.update((IParamFormula) this.mParam);
			} else if (this.mParam instanceof IParamFile) {
				this.update((IParamFile) this.mParam);
			} else if (this.mParam instanceof IParamDirectory) {
				this.update((IParamDirectory) this.mParam);
			} else {
				/* TODO exception ? */
				// throw new Exception("Invalid Parameter Type");
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Parameter specific Flags.
	 * 
	 * @param param
	 *            - Parameter
	 */
	private void update(IParameter param) {
		String str;
		this.mModified.setVisible(true);
		this.mReadOnly.setVisible(true);
		this.mDataEmpty.setVisible(false);
		this.mUsed.setVisible(true);
		this.mUpdateRequired.setVisible(false);
		this.mUncertain.setVisible(true);
		this.mIncomplete.setVisible(false);
		this.mError.setVisible(true);
		this.mPrint.setVisible(true);
		this.mPrint.setEnabled(false);
		this.mDataBase.setVisible(false);
		if (param != null) {
			try {
				this.mModified.setSelected(param.getModified());
				this.mUncertain.setSelected(param.getUncertain());
				this.mUsed.setSelected(param.getUsed()); /* remove ? */
				this.mReadOnly.setSelected(param.getReadOnly());
				this.mError.setSelected(param.getError());
				this.mPrint.setSelected(param.getPrint());
				this.mPrint.setEnabled(true);

				str = "<html>Used by:";
				for (IParameter p : param.getUsedBy()) {
					str += "<br>" + p.getIdentifier();
				}
				str += "</html>";
				this.mUsed.setToolTipText(str);

				str = "<html>Set by:";
				for (IParameter p : param.getSetBy()) {
					str += "<br>" + p.getIdentifier();
				}
				str += "</html>";
				this.mReadOnly.setToolTipText(str);

			} catch (RemoteException e) {
				this.mModified.setSelected(true);
				this.mUncertain.setSelected(true);
				this.mUsed.setSelected(true);
				this.mReadOnly.setSelected(true);
				this.mError.setSelected(true);
				this.mPrint.setSelected(true);
				e.printStackTrace();
			}
		}

	}

	/**
	 * Project specific Flags.
	 * 
	 * @param project
	 *            - Project
	 * @throws RemoteException
	 */
	private void update(IProject project) throws RemoteException {
		this.update((IParamContainer) project);
	}

	/**
	 * ParamField specific Flags.
	 * 
	 * @param pField
	 *            - ParamField
	 * @throws RemoteException
	 */
	private void update(IParamField pField) throws RemoteException {
		this.mDataBase.setVisible(true);
		this.mDataBase.setSelected(pField.getDbId() != -1);
		this.mDataEmpty.setVisible(true);
		this.mDataEmpty.setSelected(pField.getData() instanceof TNull);
	}

	/**
	 * ParamFunction specific Flags.
	 * 
	 * @param pFunction
	 *            - ParamFunction
	 * @throws RemoteException
	 */
	private void update(IParamFunction pFunction) throws RemoteException {
		this.update((IParamContainer) pFunction);
		this.mUpdateRequired.setVisible(true);
		this.mUpdateRequired.setSelected(pFunction.getUpdateRequired());
	}

	/**
	 * ParamFormula specific Flags.
	 * 
	 * @param pFormula
	 *            - ParamFormula
	 * @throws RemoteException
	 */
	private void update(IParamFormula pFormula) throws RemoteException {
		this.update((IParamField) pFormula);
		this.mUpdateRequired.setVisible(true);
		this.mUpdateRequired.setSelected(pFormula.getUpdateRequired());
	}

	/**
	 * ParamContainer specific Flags.
	 * 
	 * @param pContainer
	 *            - ParamContainer
	 * @throws RemoteException
	 */
	private void update(IParamContainer pContainer) throws RemoteException {
		this.mIncomplete.setVisible(true);
		this.mIncomplete.setSelected(pContainer.getIncomplete());
	}

	/**
	 * ParamFile specific Flags.
	 * 
	 * @param pFile
	 *            - ParamFile
	 */
	private void update(IParamFile pFile) {
		/* TODO */
	}

	/**
	 * ParamDirectory specific Flags.
	 * 
	 * @param pDirectory
	 *            - ParamDirectory
	 */
	private void update(IParamDirectory pDirectory) {
		/* TODO */
	}
}
