/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.component;

import java.awt.Desktop;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;

import at.lcm.bu21.magopt.structure.IComponent;
import magopt.gui.view.ParamBaseScroll;
import magopt.gui.web.WebPanel;

/**
 * Display help-content of component
 * 
 * @author Werner Koppelsttter
 */
public class ComponentHelpView extends ParamBaseScroll {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** ParamComponent to display */
	private IComponent _component;

	/** Pane to dispaly html (browser) */
	// private JEditorPane _browser;
	private WebPanel _browser;

	private boolean isempty = true;

	/**
	 * Create component view
	 */
	public ComponentHelpView() {

		// browser
		// this._browser = new JEditorPane();
		// this._browser.setEditable(false);
		this._browser = new WebPanel();
		this.add(this._browser);

		// open in browser
		this._browser.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent event) {
				File help;
				if (event.getClickCount() == 2) {
					try {
						help = ComponentHelpView.this._component.getHelpFile();
						if ((help != null) && help.exists() && (Desktop.isDesktopSupported())) {
							try {
								Desktop.getDesktop().browse(help.toURI());
							} catch (IOException e) {
								ComponentHelpView.this._component.getContainer()
										.info("Unable to open: " + help.getPath());
							}
						}
					} catch (RemoteException e1) {
					}
				}

			}
		});
	}

	/**
	 * Sets new ParamComponent.
	 * 
	 * @param newComponent
	 *            - ParamComponent
	 */
	public void setComponent(IComponent newComponent) {
		this._component = newComponent;
		this.update();
	}

	@Override
	protected void update() {
		File file;

		if (this._component != null) {
			try {
				file = this._component.getHelpFile();
				if ((file != null) && file.exists()) {
					// this._browser.setPage(file.toURI().toURL());
					this._browser.load(file.toURI().toURL().toString());
					this.isempty = false;
				} else {
					if (!this.isempty) {
						// this._browser.setText("");
						this._browser.loadContent("");
						this.isempty = true;
					}
				}
			} catch (Exception e1) {
				if (!this.isempty) {
					// this._browser.setText("");
					this._browser.loadContent("");
					this.isempty = true;
				}
			}
		} else {
			if (!this.isempty) {
				// this._browser.setText("");
				this._browser.loadContent("");
				this.isempty = true;
			}
		}
	}
}
