/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.docfiles;

import java.rmi.RemoteException;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.magopt.structure.DocFile;
import at.lcm.bu21.magopt.structure.IParameter;

/**
 * Table-model for DocFiles
 * 
 * @author Werner Koppelsttter
 */
public class DocFilesModel extends AbstractTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Parameter of DocFiles */
	protected IParameter param = null;

	/**
	 * Set parameter for display doc-files
	 * 
	 * @param param
	 */
	public void setParameter(IParameter param) {
		this.param = param;

		// refresh table of model
		for (TableModelListener listener : this
				.getListeners(TableModelListener.class)) {
			if ((listener instanceof JTable)
					&& this.equals(((JTable) listener).getModel())) {
				listener.tableChanged(new TableModelEvent(this));
			}
		}
	}

	@Override
	public int getColumnCount() {
		return DocFilesColumn.values().length;
	}

	@Override
	public int getRowCount() {
		int len = 0;
		if (this.param != null) {
			try {
				len = this.param.getDocFiles().length;
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}

		return len;
	}

	@Override
	public Object getValueAt(int row, int column) {
		Object obj = null;
		DocFile doc;

		if (this.param != null) {
			try {
				doc = this.param.getDocFiles()[row];
				if (column == DocFilesColumn.ICON.getValue()) {
					obj = this.param.getFileFromDocFile(doc);
				} else if (column == DocFilesColumn.NAME.getValue()) {
					obj = doc.getName();
				} else if (column == DocFilesColumn.DESCRIPTION.getValue()) {
					obj = doc.getDescription();
				} else if (column == DocFilesColumn.CHECKSUM.getValue()) {
					obj = this.param.checkDocFile(doc.getName());
				} else {
					obj = doc;
				}
			} catch (RemoteException e1) {
				e1.printStackTrace();
			}
		}

		return obj;
	}

	@Override
	public String getColumnName(int column) {
		for (DocFilesColumn e : DocFilesColumn.values()) {
			if (column == e.getValue()) {
				return e.getName();
			}
		}
		return null;
	}

	@Override
	public Class<?> getColumnClass(int column) {
		Class<?> cl = String.class;
		if (column == DocFilesColumn.CHECKSUM.getValue()) {
			return Boolean.class;
		}

		return cl;
	}

}
