/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.structure.database;

/**
 * Represent a property-object form database
 * 
 * @author Werner Koppelsttter
 * 
 * @param <T>
 *            Type of value
 */
public class MaterialProperty<T> {

	/** Name of property */
	protected String name;
	/** Description of property */
	protected String description;
	/** Group of property */
	protected String group;
	/** Value of property */
	protected T value;
	/** Unit of property */
	protected String unit;
	/** Tex-string of property */
	protected String tex;
	/** Locked-flag of property */
	protected boolean locked;
	/** Fixed-flag of property */
	protected boolean fixed;

	/**
	 * Construct a property object
	 * 
	 * @param name
	 *            Name
	 * @param description
	 *            Description
	 * @param value
	 *            Value
	 * @param group
	 *            Group
	 * @param unit
	 *            Unit
	 * @param tex
	 *            Tex-string
	 * @param locked
	 *            Locked-flag
	 * @param fixed
	 *            Fixed-flag
	 */
	public MaterialProperty(String name, String description, String group,
			T value, String unit, String tex, boolean locked, boolean fixed) {
		this.name = name;
		this.description = description;
		this.group = group;
		this.value = value;
		this.unit = unit;
		this.tex = tex;
		this.locked = locked;
		this.fixed = fixed;
	}

	/**
	 * Get name of property
	 * 
	 * @return Name
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Description of property
	 * 
	 * @return Description
	 */
	public String getDescription() {
		return this.description;
	}

	/**
	 * Group of property
	 * 
	 * @return Group
	 */
	public String getGroup() {
		return this.group;
	}

	/**
	 * Get value of property
	 * 
	 * @return Value
	 */
	public T getValue() {
		return this.value;
	}

	/**
	 * Get unit of property
	 * 
	 * @return Unit
	 */
	public String getUnit() {
		return this.unit;
	}

	/**
	 * Get tex-string of property
	 * 
	 * @return Tex-string
	 */
	public String getTex() {
		return this.tex;
	}

	/**
	 * Get locked-flag of property
	 * 
	 * @return Locked-flag
	 */
	public boolean getLocked() {
		return this.locked;
	}

	/**
	 * Get fixed-flag of property
	 * 
	 * @return Locked-flag
	 */
	public boolean getFixed() {
		return this.fixed;
	}
}
