/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.utils;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.management.ManagementFactory;
import java.util.Arrays;

/**
 * Start extern process and get process identifier
 * 
 * @author Werner Koppelsttter
 */
public class ExecuteStarter {

	/**
	 * Execute program with given parameter
	 * 
	 * @param args
	 *            Parameter list to execute. First parameter is execution
	 *            directory
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		ProcessBuilder builder;
		final Process proc;

		// write process identifier to outputstream
		System.out.println(
				ManagementFactory.getRuntimeMXBean().getName().split("@")[0]);

		// run process
		builder = new ProcessBuilder(Arrays.copyOfRange(args, 1, args.length));
		if (args[0].compareTo("@") != 0) {
			builder.directory(new File(args[0]));
		}
		proc = builder.start();

		// bypass output
		new Thread() {
			@Override
			public void run() {
				InputStream in = proc.getInputStream();
				int val;
				try {
					while ((val = in.read()) != -1) {
						System.out.write(val);
					}
				} catch (IOException e) {
					System.err.println(e.getMessage());
					e.printStackTrace();
				}

			}
		}.start();
		// bypass error
		new Thread() {
			@Override
			public void run() {
				InputStream in = proc.getErrorStream();
				int val;
				try {
					while ((val = in.read()) != -1) {
						System.err.write(val);
					}
				} catch (IOException e) {
					System.err.println(e.getMessage());
					e.printStackTrace();
				}

			}
		}.start();
		// bypass input
		new Thread() {
			@Override
			public void run() {
				OutputStream out = proc.getOutputStream();
				int val;
				try {
					while ((val = System.in.read()) != -1) {
						out.write(val);
					}
				} catch (IOException e) {
					System.err.println(e.getMessage());
					e.printStackTrace();
				}

			}
		}.start();

		proc.waitFor();

		System.exit(proc.exitValue());

	}
}
