/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.utils;

import java.io.File;
import java.util.Hashtable;
import java.util.Map;
import java.util.UUID;

import at.lcm.bu21.general.dtypes.TNumeric;
import at.lcm.bu21.magopt.structure.Function;
import at.lcm.bu21.magopt.structure.Parameter;
import at.lcm.bu21.magopt.structure.Project;

/**
 * @author gweidenholzer
 * 
 */
public class ProjectTester extends Thread {

	/** reference Project handle */
	Project refProj;
	/** Log String */
	String log = "";
	/** flag: all tests ok? */
	boolean passedAllTests = true;

	/**
	 * 
	 * @param f
	 * @throws Exception
	 */
	public ProjectTester(File f) throws Exception {
		this.refProj = Project.load(f);
		if (!new File(this.refProj.getFile()).exists()) {
			throw new Exception("illegal mop-File!");
		}
	}

	/**
	 * 
	 * @return Log String
	 */
	public String getLog() {
		return this.log;
	}

	public boolean passed() {
		return this.passedAllTests;
	}

	@Override
	public void run() {

		try {
			this.log = "";

			// ////////////////////////////////////////////////////////////
			// Create test-project
			File projFile;
			do {
				String str = UUID.randomUUID().toString() + ".mop";
				projFile = new File(
						(new File(this.refProj.getFile())).getParentFile(), str);

			} while (projFile.exists());

			Project testProj = new Project("Testproject", "", projFile);
			this.refProj.cloneParam(testProj, this.refProj.getIndex());

			// ////////////////////////////////////////////////////////////
			// reset functions
			Parameter[] functions = testProj.getParameters(Function.class);
			for (Parameter param : functions) {
				Function fnc = (Function) param;
				fnc.reset();
			}
			testProj.setError(false);
			testProj.refresh();

			// ////////////////////////////////////////////////////////////
			// regenerate test project

			Map<String, Object> args = new Hashtable<String, Object>();
			args.put("RunSim", true);
			args.put("RemoteCompute", true);
			args.put("Optimize", true);
			do {
				testProj.regenerate(args);
				Thread.sleep(1000);
			} while (!testProj.getError() && testProj.getIncomplete());

			for (Parameter paramRef : this.refProj.getAnalysisParams()) {

				boolean passed = true;

				if (paramRef.getComment().startsWith("$assert$")) {
					Parameter paramTest = testProj.getObject(paramRef
							.getIdentifier());

					String[] testStr = paramRef.getComment().split("\\$");

					if (testStr[2].compareTo("equals") == 0) {
						if (!paramRef.getData().equals(paramTest.getData())) {
							passed = false;
						}

					} else if (testStr[2].compareTo("range") == 0) {

						double lower = Double.parseDouble(testStr[3]);
						double upper = Double.parseDouble(testStr[4]);
						double dataTest = ((TNumeric) paramTest.getData())
								.getReal(0, 0);

						if ((dataTest < lower) || (dataTest > upper)) {
							passed = false;
						}
					}

					if (passed) {
						this.log = this.log + "PASSED:";
					} else {
						this.log = this.log + "FAILED:";
						this.passedAllTests = false;
					}

					this.log = this.log + paramRef.getIdentifier() + " - ";
					this.log = this.log + paramTest.getComment() + " ("
							+ paramTest.getData() + ")\n";

				}
			}
			System.out.println("\n\n TEST RESULTS:\n\n" + this.log);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
