/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * ProjectTesterGui.java
 *
 * Created on 31.05.2010, 10:03:47
 */
package magopt.utils;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.table.DefaultTableModel;

import at.lcm.bu21.magopt.structure.Project;

/**
 * 
 * @author gweidenholzer
 */
public class ProjectTesterGui extends javax.swing.JFrame {

	public static void main(String[] args) {
		if (args.length == 0) {
			new ProjectTesterGui();
		} else {
			new ProjectTesterGui(new File(args[0]));
		}
	}

	/** Creates new form ProjectTesterGui */
	public ProjectTesterGui(File directory) {
		this.initComponents();

		DefaultTableModel mdl = (DefaultTableModel) this.jTableTests.getModel();
		this.addFiles(mdl, directory);
		this.setDefaultCloseOperation(javax.swing.WindowConstants.HIDE_ON_CLOSE);
		this.setVisible(true);

	}

	public ProjectTesterGui() {
		this(new File(Project.getLibraryPath()));
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		this.jScrollPane1 = new javax.swing.JScrollPane();
		this.jTableTests = new javax.swing.JTable();
		this.buttonRun = new javax.swing.JButton();

		this.setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

		this.jTableTests.setModel(new javax.swing.table.DefaultTableModel(
				new Object[][] {}, new String[] { "Test File", "Enable Test",
						"Status" }) {

			Class[] types = new Class[] { java.lang.String.class,
					java.lang.Boolean.class, java.lang.String.class };
			boolean[] canEdit = new boolean[] { false, true, false };

			@Override
			public Class getColumnClass(int columnIndex) {
				return this.types[columnIndex];
			}

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return this.canEdit[columnIndex];
			}
		});
		this.jScrollPane1.setViewportView(this.jTableTests);
		this.jTableTests.getColumnModel().getColumn(0).setMinWidth(400);

		this.buttonRun.setText("Run Tests");
		this.buttonRun.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				ProjectTesterGui.this.buttonRunActionPerformed(evt);
			}
		});

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				this.getContentPane());
		this.getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(
														this.buttonRun,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														180,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addComponent(
														this.jScrollPane1,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														669, Short.MAX_VALUE))
								.addContainerGap()));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						javax.swing.GroupLayout.Alignment.TRAILING,
						layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(this.jScrollPane1,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										380, Short.MAX_VALUE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.buttonRun).addContainerGap()));

		this.pack();
	}// </editor-fold>//GEN-END:initComponents

	private void buttonRunActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_buttonRunActionPerformed

		DefaultTableModel mdl = (DefaultTableModel) this.jTableTests.getModel();

		for (int k = 0; k < mdl.getRowCount(); k++) {

			Boolean enable = (Boolean) mdl.getValueAt(k, 1);

			if (enable) {

				File f = new File((String) mdl.getValueAt(k, 0));
				try {

					mdl.setValueAt("test running...", k, 2);
					ProjectTester t = new ProjectTester(f);

					t.start();
					while (t.isAlive()) {
						Thread.sleep(1000);
					}

					if (t.passed()) {
						mdl.setValueAt("passed", k, 2);
					} else {
						mdl.setValueAt("failed!", k, 2);
					}

				} catch (Exception ex) {
					Logger.getLogger(ProjectTesterGui.class.getName()).log(
							Level.SEVERE, null, ex);
				}

			}

		}

	}// GEN-LAST:event_buttonRunActionPerformed

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton buttonRun;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JTable jTableTests;

	// End of variables declaration//GEN-END:variables

	private void addFiles(DefaultTableModel mdl, File file) {

		if (file.isDirectory()) {
			for (File f : file.listFiles()) {
				this.addFiles(mdl, f);
			}
		} else if (file.getName().matches(".+\\$\\$\\.mopt?")) {
			mdl.addRow(new Object[0]);
			int row = mdl.getRowCount() - 1;
			mdl.setValueAt(file.getAbsolutePath(), row, 0);
			mdl.setValueAt(false, row, 1);
			mdl.setValueAt("", row, 2);
		}

	}
}
