/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 247 $
 * $Date: 2020-03-19 10:13:52 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.optim.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URL;

import javax.swing.JApplet;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.WindowConstants;

import at.lcm.bu21.general.utils.Pair;
import at.lcm.bu21.magopt.structure.Project;
import at.lcm.bu21.optim.core.Archive;
import at.lcm.bu21.optim.core.Problem;
import at.lcm.bu21.optim.gui.project.PanelPlots;
import magopt.utils.MopFile;

/**
 * Test Applet and Application for Optimizer
 * 
 * @author Werner Koppelsttter
 */
public class OptimizerApplet extends JApplet {

	private JTabbedPane plots;

	public static void main(String[] args) throws Exception {

		if (args.length >= 3) {
			MopFile mop = new MopFile(args[0]);
			Project project = Project.load(mop);
			Problem problem = Archive.load(mop, args[1]).readProblem(args[2]);

			// export file
			if (args.length >= 4) {
				Pair<Project, Problem> pair = new Pair<Project, Problem>(project, problem);
				ObjectOutputStream os = new ObjectOutputStream(new FileOutputStream(args[3]));
				os.writeObject(pair);
				os.close();
			}

			// display in frame
			JFrame frame = new JFrame();
			frame.setLayout(new BorderLayout());
			frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
			new OptimizerApplet().initialize(frame, project, problem);
			frame.setSize(1000, 800);
			frame.setVisible(true);
		} else {
			System.out.println("Call:\n\tOptimizerApplet <file> <archive> <problem> [<out>]");
		}

	}

	@Override
	public void init() {
		URL url;

		String file = this.getParameter("Pair");

		try {
			url = new URL(this.getCodeBase() + file);
			ObjectInputStream is = new ObjectInputStream(url.openStream());
			Pair<Project, Problem> pair = (Pair<Project, Problem>) is.readObject();
			is.close();
			this.initialize(this, pair.getFirst(), pair.getSecond());
		} catch (Exception e) {
			StringWriter str = new StringWriter();
			e.printStackTrace(new PrintWriter(str));
			StringBuffer buf = str.getBuffer();
			buf.insert(0, "ERROR:\n" + e.getMessage() + "\n");
			this.add(new JTextArea(buf.toString()));
		}
	}

	private void initialize(final Container comp, Project project, Problem problem) throws Exception {
		// Execute a job on the event-dispatching thread; creating this applet's
		// GUI.
		// try {
		// SwingUtilities.invokeAndWait(new Runnable() {
		// @Override
		// public void run() {
		// MopFile mop = new MopFile("Optimizer_test_DTLZ1_Java.mop");
		// MopFile mop = new MopFile(
		// "U:\\test\\_Demo2\\Optimizer_test_DTLZ1_Java.mop".replace("\\",
		PanelPlots panelPlots = new PanelPlots(problem);

		this.plots = new JTabbedPane();
		comp.add(this.plots);
		for (int k = 0; k < panelPlots.getNumFixPlots(); k++) {
			this.plots.addTab(panelPlots.getFixPlotName(k), panelPlots.getFixPlot(k).getPanel());
		}
		for (int k = 0; k < panelPlots.getNumUserPlots(); k++) {
			this.plots.addTab(panelPlots.getUserPlotName(k), panelPlots.getUserPlot(k).getPanel());
		}

		panelPlots.refresh();
		// comp.add(new JLabel(baseProject.getName()));

		// }
		// });
		// } catch (Exception e) {
		// System.err.println("createGUI didn't complete successfully");
		// this.e.printStackTrace();
		// }

	}
}
