#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 10062 $
$LastChangedDate:: 2020-11-20 13:22:02 +0100#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import shutil
from symspace.generic_gui_qt import GUIFormThread
from symspace.osutils.pathutils import Paths
from symspace.console.script_utils import ScriptUtils
from symspace.wrapper.symwrapper import SymProject


def main(project: SymProject):
    if project is None:
        start_dir = Paths.getSyMSpaceRoot()
    else:
        start_dir = project.getDirectory() + '\\'
    form_title = 'Create new Python Script'
    form_elements = [('Script name', 'browse_file', start_dir, 'Project files (*.ipynb)'),
                     ('Open in editor', 'checkbox'),
                     ('Open folder', 'checkbox'),
                     ]
    form = GUIFormThread(form_title, form_elements)
    form.start()

    if form.accept:
        script = form.result['Script name']
        openeditor = form.result['Open in editor']
        openfolder = form.result['Open folder']

        if not script.endswith('.py'):
            script += '.py'

        template = os.path.join(Paths.getSyMSpaceRoot(), 'System', 'Python', 'resources', 'NewScript',
                                'new_script.py')
        shutil.copyfile(template, script)

        if openeditor:
            ScriptUtils.openExplorer(script)
        if openfolder:
            ScriptUtils.openExplorer(os.path.dirname(script))

    else:
        print('Cancelled by user')


if __name__ == '__main__':
    """
    This script creates a new Python script from a template rile
    """

    _params = locals()
    _project = ScriptUtils.getProject(_params)
    ScriptUtils.run(main, _project)
