# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10062 $
$LastChangedDate:: 2020-11-20 13:22:02 +0100#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import pandas as pd
from symspace.console.script_utils import ScriptUtils
from symspace.generic_gui_qt import GUIFormThread
from symspace.wrapper.symoptutils import SyMOptUtils
from symspace.wrapper.symoptwrapper import SymArchive


def main(params: dict):
    project = ScriptUtils.selectProjectFromParams(params)
    if project is not None:
        project_file = project.getFile(get_abs_path=True)
        default_excel_file = os.path.join(os.path.dirname(project_file), 'pareto.xlsx')

        archive_list = SymArchive.listArchives(project_file)

        if len(archive_list) == 0:
            raise Exception('The selected project does not contain optimization archives!')

        form_title = 'Add individuals to archive'
        form_elements = [
            ('Project', 'label', project_file),
            ('Archive Name', 'combobox', archive_list),
            ('Select Excel file', 'browse_file', default_excel_file, 'Excel Files (*.xlsx *.xls)'),
            ('Map Short Identifier', 'checkbox', True)
        ]

        form = GUIFormThread(form_title, form_elements)

        form.start()
        if form.accept:
            file_path = form.result['Select Excel file']
            map_ids = form.result['Map Short Identifier']
            archive_name = form.result['Archive Name']

            archive = SymArchive.load(project_file, archive_name)

            df = pd.read_excel(file_path)
            df = SyMOptUtils.renameDataFrame(archive, df, map_ids)

            SyMOptUtils.addIndividuals2Archive(archive, df)

            print('Update finished')
        else:
            print('Cancelled by user')


if __name__ == '__main__':
    """
    This script add individuals to the selected optimization archive
    
    1. Option: load individuals from an Excel file (e.g. Pareto-Front excel export)

    This script needs to be executed in the SyMSpace-Optimizer Python Console.
    """
    _params = locals()
    ScriptUtils.run(main, _params)
