# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10096 $
$LastChangedDate:: 2021-01-21 16:08:58 +0100#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
from symspace.console.script_utils import ScriptUtils
from symspace.wrapper.symwrapper import SymProject, SymContainer
from symspace.generic_gui_qt import GUIFormThread


def main(parameter: SymContainer):
    default_path = ''
    max_files = 5
    form_title = 'Register file for {}'.format(parameter.getName())
    form_elements = []
    for i in range(1, max_files + 1):
        form_elements.append(('Select file {:d}'.format(i), 'browse_file', default_path))

    form = GUIFormThread(form_title, form_elements)

    form.start()

    if form.accept:
        try:
            component = parameter.getComponent()
        except AttributeError:
            parameter.addComponentProperty()
            component = parameter.getComponent()

        if component is None:
            parameter.addComponentProperty()

        for i in range(1, max_files + 1):
            file = form.result['Select file {:d}'.format(i)]
            if os.path.isfile(file):
                component.addLinkedFile(file)
                print('Registered: {}'.format(file))
        print('Script finished successfully')
    else:
        print('Cancelled by user')


if __name__ == '__main__':
    """
    This script registers a file to a symspace project/container
    The files to register can be selected from the GUI.

    This script needs to be executed in the SyMSpace Python Console.
    """

    _params = locals()
    _project = ScriptUtils.getProject(_params)
    _parameter = ScriptUtils.getParameter(_params)

    if _project is not None:
        ScriptUtils.run(main, _parameter)
    else:
        ScriptUtils.showNoProjectWarning()
