#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 8934 $
$LastChangedDate:: 2019-02-18 15:50:09 +0100#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import os
from tkinter import filedialog
from tkinter import *


class GUI(Frame):

    # Initialization of the Frame
    def __init__(self, title, gui_elements, master=None, **kw):
        super().__init__(master, **kw)
        self.master.title(title)
        self.grid()
        self.font = ('Arial', 10, 'roman')
        width = 50

        self.results = {}
        self.entry = {}
        row = 0

        for elem in gui_elements:
            label = Label(self, text=elem[1], font=self.font)
            label.grid(row=row, column=0, sticky='w')

            self.entry[elem[0]] = Entry(self, font=self.font, width=width)
            self.entry[elem[0]].grid(row=row, column=1, padx=5, pady=5)
            self.entry[elem[0]].insert(0, elem[2])
            self.results[elem[0]] = ''
            row += 1

        self.ok_button = Button(self, text='OK', command=self.okButtonAction, font=self.font)
        self.ok_button.grid(row=row + 1, column=0, padx=5, pady=5, sticky='w')

        self.cancel_button = Button(self, text='Cancel', command=self.cancelButtonAction, font=self.font)
        self.cancel_button.grid(row=row + 1, column=1, padx=5, pady=5, sticky='w')

    def cancelButtonAction(self):
        self.destroy()
        self.master.destroy()
        self.quit()
        self.master.quit()

    def okButtonAction(self):
        for key in self.entry.keys():
            self.results[key] = self.entry[key].get()
        self.destroy()
        self.master.destroy()
        self.quit()
        self.master.quit()


def guiDialog(title, elements):
    """
    Define a GUI dialog

    Parameters
    ----------
    title : str
        Title of the dialog.

    elements : list of tuples
        Definition of GUI text fields
        (var, label, init_value)

        var : str
            Name of variable
        label : str
            Appears as label of text field
        init_value : str
            Initial value of the text field

    Returns
    -------
        result : dict
            Contains var and corresponding values if OK was pres
    """
    # root = Tk()
    # app = GUI(title, elements, master=root)
    app = GUI(title, elements)
    app.mainloop()
    res = app.results
    return res
