#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10096 $
$LastChangedDate:: 2021-01-21 16:08:58 +0100#$

This file is part of SyMSpace.

Copyright (c) 2019, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import tkinter as tk
from tkinter import ttk
from tkinter import messagebox
from symspace.osutils.pathutils import Paths


class ProgressBarTk:

    def __init__(self, title, maximum):
        self.__running = True
        self.__window = tk.Tk()
        self.__window.protocol("WM_DELETE_WINDOW", self.__on_closing)
        # self.__window.iconphoto(False, tk.PhotoImage(
        #     file=os.path.join(symspace_home, 'System', 'Icon', 'Icon_128x128.png')))  # for PNG icon
        self.__window.iconbitmap(os.path.join(Paths.getSyMSpaceRoot(), 'System', 'sym.ico'))  # for ICO icon
        self.__window.title(title)
        self.__pb = ttk.Progressbar(self.__window, orient="horizontal", length=350, mode="determinate")
        self.__pb.pack()
        self.maximum = maximum
        self.setValue(0)

    @property
    def maximum(self):
        self.__checkRunning()
        return self.__pb["maximum"]

    @maximum.setter
    def maximum(self, value: int):
        self.__checkRunning()
        self.__pb["maximum"] = value
        self.__pb.update()

    def setValue(self, value: int):
        self.__checkRunning()
        self.__pb["value"] = value
        self.__pb.update()

    def incrementValue(self, increment: int = 1):
        self.__checkRunning()
        self.__pb["value"] += increment
        self.__pb.update()

    def getValue(self) -> int:
        self.__checkRunning()
        return self.__pb["value"]

    def close(self):
        self.__checkRunning()
        self.__pb.destroy()
        self.__window.destroy()
        self.__pb.quit()
        self.__window.quit()

    def __on_closing(self):
        if messagebox.askokcancel("Quit", "Do you want to quit?"):
            self.close()
            self.__running = False

    def __checkRunning(self):
        if not self.__running:
            raise Exception('Execution has been aborted')


if __name__ == '__main__':
    import time as _time

    _pb_tk = ProgressBarTk('Demo', 100)

    for i in range(_pb_tk.maximum):
        _pb_tk.setValue(i)
        _time.sleep(1e-1)

    _pb_tk.close()
