#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 9194 $
$LastChangedDate:: 2019-06-12 18:24:37 +0200#$

This file is part of SyMSpace.
@author: mkitzberger

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

if __name__ == '__main__':

    import numpy as np

    """Script that shows basic plotting functionality"""
    try:
        from symspace.wrapper.symplot import SymPlot2Line, SymPlot2Bar, SymPlot2Scatter, SymLinkPlot, SymSubPlot, SymPlot3Surface

        # Plot a sine from numpy array
        print('# Running Script!!!')
        phi = np.linspace(0, 360, 1000)
        handle = SymPlot2Line()
        handle.addSignal('sine', phi, np.sin(phi * np.pi / 180))
        handle.setLegend(True)
        handle.show()
        # Plot a cosine from PyTPeriodicMatrix
        from lcm.utils.pytperiodic import PyTPeriodicMatrix

        handle.addSignal('cosine', PyTPeriodicMatrix(1, 1, 0))
        # Plot one PyTPeriodicMatrix over annother
        x = PyTPeriodicMatrix(1, 1, 0)
        y = PyTPeriodicMatrix(1, 1, -np.pi / 2)
        handle = SymPlot2Line()
        handle.addSignal('circle', x, y)
        handle.setLegend(True)
        handle.show()

        ''' create linked subplots '''
        n_rows = 2
        n_cols = 2
        sym_plots = np.empty((n_rows, n_cols), np.dtype(object))
        sym_link = SymLinkPlot()
        for row in range(n_rows):
            for col in range(n_cols):
                sym_plots[row, col] = SymPlot2Scatter()
                sym_plots[row, col].addSignal('random values', np.random.rand(1, 4), np.random.rand(1, 4))
                sym_plots[row, col].setLegend(True)
                sym_link.addPlot(sym_plots[row, col])

        sym_subplot = SymSubPlot(n_rows, n_cols)
        sym_subplot.setPlots(sym_plots)

        surf3D = SymPlot3Surface()
        x = np.linspace(0, 100)
        y = np.linspace(0, 100)
        z = np.random.rand(x.size, y.size) * 100
        surf3D.addSignal('surf3D', x, y, z)
        sym_subplot.setPlot(0, 0, surf3D)
        sym_subplot.show()

    except Exception as e:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        print('\n{} in Script on line {}:'.format(exc_type.__name__, exc_traceback.tb_lineno))
        print(exc_value)
