/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 10128 $
 * $Date: 2021-03-10 11:08:49 +0100 (Wed, 10 Mar 2021) $
 */
package at.lcm.symspace.rest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.json.simple.JSONObject;

import at.lcm.bu21.magopt.structure.ParamCollector;
import at.lcm.bu21.magopt.structure.ParamComboBox;
import at.lcm.bu21.magopt.structure.ParamContainer;
import at.lcm.bu21.magopt.structure.ParamDirectory;
import at.lcm.bu21.magopt.structure.ParamField;
import at.lcm.bu21.magopt.structure.ParamFile;
import at.lcm.bu21.magopt.structure.ParamFormula;
import at.lcm.bu21.magopt.structure.ParamFunction;
import at.lcm.bu21.magopt.structure.ParamJavaFunction;
import at.lcm.bu21.magopt.structure.ParamLinkedField;
import at.lcm.bu21.magopt.structure.ParamMaterial;
import at.lcm.bu21.magopt.structure.ParamMatlabFunction;
import at.lcm.bu21.magopt.structure.ParamPythonFunction;
import at.lcm.bu21.magopt.structure.Parameter;
import at.lcm.bu21.magopt.structure.Project;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.OPTIONS;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;

@Path("tree")
public class TreeStructure {

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public String getTreeStucture(@QueryParam("projectid") String projectId) {
		return JSONObject.toJSONString(getTreeStructure(ProjectDetails.getProject(projectId)));
	}

	@OPTIONS
	@Produces(MediaType.APPLICATION_JSON)
	public Response optionsFoListResource() {
		return Response.status(200).header("Allow", "POST, PUT, GET").header("Content-Type", MediaType.APPLICATION_JSON)
				.header("Content-Length", "0").build();
	}

	@GET
	@Path("entry")
	@Produces(MediaType.APPLICATION_JSON)
	public String getTreeContainer(@QueryParam("projectid") String projectId, @QueryParam("pid") String pid) {
		return JSONObject.toJSONString(
				getContainerStructure((ParamContainer) ProjectDetails.getProject(projectId).getParamByPid(pid)));
	}

	@OPTIONS
	@Path("entry")
	@Produces(MediaType.APPLICATION_JSON)
	public Response optionsFoListResourceEntry() {
		return Response.status(200).header("Allow", "POST, PUT, GET").header("Content-Type", MediaType.APPLICATION_JSON)
				.header("Content-Length", "0").build();
	}

	private Map<String, Object> getTreeStructure(ParamContainer container) {
		HashMap<String, Object> map = new HashMap<String, Object>();
		ArrayList<Map<String, Object>> paramlist;

		map.put("name", container.getName());
		map.put("desripction", container.getDescription());
		map.put("identifier", container.getIdentifier());
		map.put("type", getType(container));
		map.put("link", getContainerUrl(container));
		map.put("error", container.getError());
		map.put("uncertain", container.getUncertain());
		map.put("incomplete", container.getIncomplete());

		paramlist = new ArrayList<Map<String, Object>>();
		map.put("param", paramlist);
		for (Parameter p : container.getParam()) {
			if (p instanceof ParamContainer) {
				paramlist.add(getTreeStructure((ParamContainer) p));
			}
		}

		return map;
	}

	String getContainerUrl(ParamContainer container) {
		StringBuilder sb = new StringBuilder();

		if (container != null) {
			sb.append(RESTServer.getBaseUri());
			sb.append("/tree/entry?projectid=");
			sb.append(ProjectDetails.getProjectId());
			sb.append("&pid=");
			sb.append(container.getPId());
		}

		return sb.toString();
	}

	private Map<String, Object> getContainerStructure(ParamContainer container) {
		Map<String, Object> map, map2;
		ArrayList<Map<String, Object>> paramlist;

		map = getParameterStructure(container);
		paramlist = new ArrayList<Map<String, Object>>();
		map.put("param", paramlist);
		for (Parameter p : container.getParam()) {
			map2 = getParameterStructure(p);
			paramlist.add(map2);
			if (p instanceof ParamContainer) {
				map2.put("link", getContainerUrl((ParamContainer) p));
			}
		}

		return map;
	}

	private Map<String, Object> getParameterStructure(Parameter param) {
		HashMap<String, Object> map = new HashMap<String, Object>();

		map.put("name", param.getName());
		map.put("desripction", param.getDescription());
		map.put("identifier", param.getIdentifier());
		map.put("type", getType(param));
//		map.put("linkParam", getParameterUrl(param));

		return map;
	}

	String getParameterUrl(Parameter param) {
		StringBuilder sb = new StringBuilder();

		if (param != null) {
			sb.append(RESTServer.getBaseUri());
			sb.append(getType(param));
			sb.append("?projectid=");
//			sb.append("/");
			sb.append(ProjectDetails.getProjectId());
//			sb.append("/");
			sb.append("&pid=");
			sb.append(param.getPId());
		}

		return sb.toString();

	}

	String getType(Parameter param) {
		String type;

		if (param instanceof Project) {
			type = "project";
		} else if (param instanceof ParamJavaFunction) {
			type = "javafunction";
		} else if (param instanceof ParamPythonFunction) {
			type = "pythonfunction";
		} else if (param instanceof ParamMatlabFunction) {
			type = "matlabfunction";
		} else if (param instanceof ParamFunction) {
			type = "function";
		} else if (param instanceof ParamCollector) {
			type = "collector";
		} else if (param instanceof ParamMaterial) {
			type = "material";
		} else if (param instanceof ParamContainer) {
			type = "container";
		} else if (param instanceof ParamFormula) {
			type = "formula";
		} else if (param instanceof ParamLinkedField) {
			type = "linkedfield";
		} else if (param instanceof ParamComboBox) {
			type = "combobox";
		} else if (param instanceof ParamField) {
			type = "field";
		} else if (param instanceof ParamFile) {
			type = "file";
		} else if (param instanceof ParamDirectory) {
			type = "directory";
//		} else if (param instanceof Parameter) {
//			type = "parameter";
		} else {
//			type = "";
			type = "parameter";
		}

		return type;
	}

}
